@@ -147,66 +147,6 @@ check_directories_created(const char *cache_dir)
    expect_true(sub_dirs_created, "create sub dirs");
 }
 
-#define CACHE_TEST_TMP "./cache-test-tmp"
-
-static void
-test_disk_cache_create(void)
-{
-   struct disk_cache *cache;
-   int err;
-
-   /* Before doing anything else, ensure that with
-    * MESA_GLSL_CACHE_DISABLE set, that disk_cache_create returns NULL.
-    */
-   setenv("MESA_GLSL_CACHE_DISABLE", "1", 1);
-   cache = disk_cache_create("test", "make_check");
-   expect_null(cache, "disk_cache_create with MESA_GLSL_CACHE_DISABLE set");
-
-   unsetenv("MESA_GLSL_CACHE_DISABLE");
-
-   /* For the first real disk_cache_create() clear these environment
-    * variables to test creation of cache in home directory.
-    */
-   unsetenv("MESA_GLSL_CACHE_DIR");
-   unsetenv("XDG_CACHE_HOME");
-
-   cache = disk_cache_create("test", "make_check");
-   expect_non_null(cache, "disk_cache_create with no environment variables");
-
-   disk_cache_destroy(cache);
-
-   /* Test with XDG_CACHE_HOME set */
-   setenv("XDG_CACHE_HOME", CACHE_TEST_TMP "/xdg-cache-home", 1);
-   cache = disk_cache_create("test", "make_check");
-   expect_null(cache, "disk_cache_create with XDG_CACHE_HOME set with"
-               "a non-existing parent directory");
-
-   mkdir(CACHE_TEST_TMP, 0755);
-   cache = disk_cache_create("test", "make_check");
-   expect_non_null(cache, "disk_cache_create with XDG_CACHE_HOME set");
-
-   check_directories_created(CACHE_TEST_TMP "/xdg-cache-home/mesa");
-
-   disk_cache_destroy(cache);
-
-   /* Test with MESA_GLSL_CACHE_DIR set */
-   err = rmrf_local(CACHE_TEST_TMP);
-   expect_equal(err, 0, "Removing " CACHE_TEST_TMP);
-
-   setenv("MESA_GLSL_CACHE_DIR", CACHE_TEST_TMP "/mesa-glsl-cache-dir", 1);
-   cache = disk_cache_create("test", "make_check");
-   expect_null(cache, "disk_cache_create with MESA_GLSL_CACHE_DIR set with"
-               "a non-existing parent directory");
-
-   mkdir(CACHE_TEST_TMP, 0755);
-   cache = disk_cache_create("test", "make_check");
-   expect_non_null(cache, "disk_cache_create with MESA_GLSL_CACHE_DIR set");
-
-   check_directories_created(CACHE_TEST_TMP "/mesa-glsl-cache-dir/mesa");
-
-   disk_cache_destroy(cache);
-}
-
 static bool
 does_cache_contain(struct disk_cache *cache, const cache_key key)
 {
@@ -222,24 +162,82 @@ does_cache_contain(struct disk_cache *cache, const cache_key key)
    return false;
 }
 
-static void
-wait_until_file_written(struct disk_cache *cache, const cache_key key)
+static void *
+cache_exists(struct disk_cache *cache)
 {
-   struct timespec req;
-   struct timespec rem;
+   uint8_t dummy_key[20];
+   char data[] = "some test data";
 
-   /* Set 100ms delay */
-   req.tv_sec = 0;
-   req.tv_nsec = 100000000;
+   if (!cache)
+      return NULL;
 
-   unsigned retries = 0;
-   while (retries++ < 20) {
-      if (does_cache_contain(cache, key)) {
-         break;
-      }
+   disk_cache_put(cache, dummy_key, data, sizeof(data), NULL);
+   disk_cache_wait_for_idle(cache);
+   return disk_cache_get(cache, dummy_key, NULL);
+}
 
-      nanosleep(&req, &rem);
-   }
+#define CACHE_TEST_TMP "./cache-test-tmp"
+
+static void
+test_disk_cache_create(void)
+{
+   struct disk_cache *cache;
+   int err;
+
+   /* Before doing anything else, ensure that with
+    * MESA_GLSL_CACHE_DISABLE set to true, that disk_cache_create returns NULL.
+    */
+   setenv("MESA_GLSL_CACHE_DISABLE", "true", 1);
+   cache = disk_cache_create("test", "make_check", 0);
+   expect_null(cache, "disk_cache_create with MESA_GLSL_CACHE_DISABLE set");
+
+   unsetenv("MESA_GLSL_CACHE_DISABLE");
+
+   /* For the first real disk_cache_create() clear these environment
+    * variables to test creation of cache in home directory.
+    */
+   unsetenv("MESA_GLSL_CACHE_DIR");
+   unsetenv("XDG_CACHE_HOME");
+
+   cache = disk_cache_create("test", "make_check", 0);
+   expect_non_null(cache, "disk_cache_create with no environment variables");
+
+   disk_cache_destroy(cache);
+
+   /* Test with XDG_CACHE_HOME set */
+   setenv("XDG_CACHE_HOME", CACHE_TEST_TMP "/xdg-cache-home", 1);
+   cache = disk_cache_create("test", "make_check", 0);
+   expect_null(cache_exists(cache), "disk_cache_create with XDG_CACHE_HOME set "
+               "with a non-existing parent directory");
+
+   mkdir(CACHE_TEST_TMP, 0755);
+   cache = disk_cache_create("test", "make_check", 0);
+   expect_non_null(cache_exists(cache), "disk_cache_create with XDG_CACHE_HOME "
+                   "set");
+
+   check_directories_created(CACHE_TEST_TMP "/xdg-cache-home/"
+                             CACHE_DIR_NAME);
+
+   disk_cache_destroy(cache);
+
+   /* Test with MESA_GLSL_CACHE_DIR set */
+   err = rmrf_local(CACHE_TEST_TMP);
+   expect_equal(err, 0, "Removing " CACHE_TEST_TMP);
+
+   setenv("MESA_GLSL_CACHE_DIR", CACHE_TEST_TMP "/mesa-glsl-cache-dir", 1);
+   cache = disk_cache_create("test", "make_check", 0);
+   expect_null(cache_exists(cache), "disk_cache_create with MESA_GLSL_CACHE_DIR"
+               " set with a non-existing parent directory");
+
+   mkdir(CACHE_TEST_TMP, 0755);
+   cache = disk_cache_create("test", "make_check", 0);
+   expect_non_null(cache_exists(cache), "disk_cache_create with "
+                   "MESA_GLSL_CACHE_DIR set");
+
+   check_directories_created(CACHE_TEST_TMP "/mesa-glsl-cache-dir/"
+                             CACHE_DIR_NAME);
+
+   disk_cache_destroy(cache);
 }
 
 static void
@@ -256,7 +254,7 @@ test_put_and_get(void)
    uint8_t one_KB_key[20], one_MB_key[20];
    int count;
 
-   cache = disk_cache_create("test", "make_check");
+   cache = disk_cache_create("test", "make_check", 0);
 
    disk_cache_compute_key(cache, blob, sizeof(blob), blob_key);
 
@@ -266,12 +264,10 @@ test_put_and_get(void)
    expect_equal(size, 0, "disk_cache_get with non-existent item (size)");
 
    /* Simple test of put and get. */
-   disk_cache_put(cache, blob_key, blob, sizeof(blob));
+   disk_cache_put(cache, blob_key, blob, sizeof(blob), NULL);
 
-   /* disk_cache_put() hands things off to a thread give it some time to
-    * finish.
-    */
-   wait_until_file_written(cache, blob_key);
+   /* disk_cache_put() hands things off to a thread so wait for it. */
+   disk_cache_wait_for_idle(cache);
 
    result = disk_cache_get(cache, blob_key, &size);
    expect_equal_str(blob, result, "disk_cache_get of existing item (pointer)");
@@ -281,12 +277,10 @@ test_put_and_get(void)
 
    /* Test put and get of a second item. */
    disk_cache_compute_key(cache, string, sizeof(string), string_key);
-   disk_cache_put(cache, string_key, string, sizeof(string));
+   disk_cache_put(cache, string_key, string, sizeof(string), NULL);
 
-   /* disk_cache_put() hands things off to a thread give it some time to
-    * finish.
-    */
-   wait_until_file_written(cache, string_key);
+   /* disk_cache_put() hands things off to a thread so wait for it. */
+   disk_cache_wait_for_idle(cache);
 
    result = disk_cache_get(cache, string_key, &size);
    expect_equal_str(result, string, "2nd disk_cache_get of existing item (pointer)");
@@ -298,7 +292,7 @@ test_put_and_get(void)
    disk_cache_destroy(cache);
 
    setenv("MESA_GLSL_CACHE_MAX_SIZE", "1K", 1);
-   cache = disk_cache_create("test", "make_check");
+   cache = disk_cache_create("test", "make_check", 0);
 
    one_KB = calloc(1, 1024);
 
@@ -322,14 +316,12 @@ test_put_and_get(void)
    disk_cache_compute_key(cache, one_KB, 1024, one_KB_key);
    one_KB_key[0] = blob_key[0];
 
-   disk_cache_put(cache, one_KB_key, one_KB, 1024);
+   disk_cache_put(cache, one_KB_key, one_KB, 1024, NULL);
 
    free(one_KB);
 
-   /* disk_cache_put() hands things off to a thread give it some time to
-    * finish.
-    */
-   wait_until_file_written(cache, one_KB_key);
+   /* disk_cache_put() hands things off to a thread so wait for it. */
+   disk_cache_wait_for_idle(cache);
 
    result = disk_cache_get(cache, one_KB_key, &size);
    expect_non_null(result, "3rd disk_cache_get of existing item (pointer)");
@@ -363,16 +355,13 @@ test_put_and_get(void)
    disk_cache_destroy(cache);
 
    setenv("MESA_GLSL_CACHE_MAX_SIZE", "1M", 1);
-   cache = disk_cache_create("test", "make_check");
+   cache = disk_cache_create("test", "make_check", 0);
 
-   disk_cache_put(cache, blob_key, blob, sizeof(blob));
-   disk_cache_put(cache, string_key, string, sizeof(string));
+   disk_cache_put(cache, blob_key, blob, sizeof(blob), NULL);
+   disk_cache_put(cache, string_key, string, sizeof(string), NULL);
 
-   /* disk_cache_put() hands things off to a thread give it some time to
-    * finish.
-    */
-   wait_until_file_written(cache, blob_key);
-   wait_until_file_written(cache, string_key);
+   /* disk_cache_put() hands things off to a thread so wait for it. */
+   disk_cache_wait_for_idle(cache);
 
    count = 0;
    if (does_cache_contain(cache, blob_key))
@@ -392,14 +381,12 @@ test_put_and_get(void)
    disk_cache_compute_key(cache, one_MB, 1024 * 1024, one_MB_key);
    one_MB_key[0] = blob_key[0];
 
-   disk_cache_put(cache, one_MB_key, one_MB, 1024 * 1024);
+   disk_cache_put(cache, one_MB_key, one_MB, 1024 * 1024, NULL);
 
    free(one_MB);
 
-   /* disk_cache_put() hands things off to a thread give it some time to
-    * finish.
-    */
-   wait_until_file_written(cache, one_MB_key);
+   /* disk_cache_put() hands things off to a thread so wait for it. */
+   disk_cache_wait_for_idle(cache);
 
    bool contains_1MB_file = false;
    count = 0;
@@ -438,7 +425,7 @@ test_put_key_and_get_key(void)
                         { 0,  1, 42, 43, 44, 45, 46, 47, 48, 49,
                          50, 55, 52, 53, 54, 55, 56, 57, 58, 59};
 
-   cache = disk_cache_create("test", "make_check");
+   cache = disk_cache_create("test", "make_check", 0);
 
    /* First test that disk_cache_has_key returns false before disk_cache_put_key */
    result = disk_cache_has_key(cache, key_a);
