@@ -33,19 +33,6 @@
 
 static pthread_mutex_t table_lock = PTHREAD_MUTEX_INITIALIZER;
 
-static uint32_t
-u32_hash(const void *key)
-{
-	return _mesa_hash_data(key, sizeof(uint32_t));
-}
-
-static bool
-u32_equals(const void *key1, const void *key2)
-{
-	return *(const uint32_t *)key1 == *(const uint32_t *)key2;
-}
-
-
 struct fd_device * kgsl_device_new(int fd);
 struct fd_device * msm_device_new(int fd);
 
@@ -90,8 +77,8 @@ out:
 
 	p_atomic_set(&dev->refcnt, 1);
 	dev->fd = fd;
-	dev->handle_table = _mesa_hash_table_create(NULL, u32_hash, u32_equals);
-	dev->name_table = _mesa_hash_table_create(NULL, u32_hash, u32_equals);
+	dev->handle_table = _mesa_hash_table_create(NULL, _mesa_hash_u32, _mesa_key_u32_equal);
+	dev->name_table = _mesa_hash_table_create(NULL, _mesa_hash_u32, _mesa_key_u32_equal);
 	fd_bo_cache_init(&dev->bo_cache, FALSE);
 	fd_bo_cache_init(&dev->ring_cache, TRUE);
 
