@@ -28,11 +28,11 @@
 #include "pipe/p_screen.h"
 #include "util/u_memory.h"
 #include "util/u_inlines.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_upload_mgr.h"
 #include "noop_public.h"
 
-DEBUG_GET_ONCE_BOOL_OPTION(noop, "GALLIUM_NOOP", FALSE)
+DEBUG_GET_ONCE_BOOL_OPTION(noop, "GALLIUM_NOOP", false)
 
 void noop_init_state_functions(struct pipe_context *ctx);
 
@@ -59,7 +59,7 @@ static void noop_destroy_query(struct pipe_context *ctx, struct pipe_query *quer
    FREE(query);
 }
 
-static boolean noop_begin_query(struct pipe_context *ctx, struct pipe_query *query)
+static bool noop_begin_query(struct pipe_context *ctx, struct pipe_query *query)
 {
    return true;
 }
@@ -69,19 +69,19 @@ static bool noop_end_query(struct pipe_context *ctx, struct pipe_query *query)
    return true;
 }
 
-static boolean noop_get_query_result(struct pipe_context *ctx,
-                                     struct pipe_query *query,
-                                     boolean wait,
-                                     union pipe_query_result *vresult)
+static bool noop_get_query_result(struct pipe_context *ctx,
+                                  struct pipe_query *query,
+                                  bool wait,
+                                  union pipe_query_result *vresult)
 {
    uint64_t *result = (uint64_t*)vresult;
 
    *result = 0;
-   return TRUE;
+   return true;
 }
 
 static void
-noop_set_active_query_state(struct pipe_context *pipe, boolean enable)
+noop_set_active_query_state(struct pipe_context *pipe, bool enable)
 {
 }
 
@@ -135,11 +135,11 @@ static struct pipe_resource *noop_resource_from_handle(struct pipe_screen *scree
    return noop_resource;
 }
 
-static boolean noop_resource_get_handle(struct pipe_screen *pscreen,
-                                        struct pipe_context *ctx,
-                                        struct pipe_resource *resource,
-                                        struct winsys_handle *handle,
-                                        unsigned usage)
+static bool noop_resource_get_handle(struct pipe_screen *pscreen,
+                                     struct pipe_context *ctx,
+                                     struct pipe_resource *resource,
+                                     struct winsys_handle *handle,
+                                     unsigned usage)
 {
    struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)pscreen;
    struct pipe_screen *screen = noop_screen->oscreen;
@@ -156,6 +156,31 @@ static boolean noop_resource_get_handle(struct pipe_screen *pscreen,
    return result;
 }
 
+static bool noop_resource_get_param(struct pipe_screen *pscreen,
+                                    struct pipe_context *ctx,
+                                    struct pipe_resource *resource,
+                                    unsigned plane,
+                                    unsigned layer,
+                                    enum pipe_resource_param param,
+                                    unsigned handle_usage,
+                                    uint64_t *value)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)pscreen;
+   struct pipe_screen *screen = noop_screen->oscreen;
+   struct pipe_resource *tex;
+   bool result;
+
+   /* resource_get_param mustn't fail. Just create something and return it. */
+   tex = screen->resource_create(screen, resource);
+   if (!tex)
+      return false;
+
+   result = screen->resource_get_param(screen, NULL, tex, 0, 0, param,
+                                       handle_usage, value);
+   pipe_resource_reference(&tex, NULL);
+   return result;
+}
+
 static void noop_resource_destroy(struct pipe_screen *screen,
                                   struct pipe_resource *resource)
 {
@@ -172,7 +197,7 @@ static void noop_resource_destroy(struct pipe_screen *screen,
 static void *noop_transfer_map(struct pipe_context *pipe,
                                struct pipe_resource *resource,
                                unsigned level,
-                               enum pipe_transfer_usage usage,
+                               unsigned usage,
                                const struct pipe_box *box,
                                struct pipe_transfer **ptransfer)
 {
@@ -296,13 +321,13 @@ static void noop_destroy_context(struct pipe_context *ctx)
    FREE(ctx);
 }
 
-static boolean noop_generate_mipmap(struct pipe_context *ctx,
-                                    struct pipe_resource *resource,
-                                    enum pipe_format format,
-                                    unsigned base_level,
-                                    unsigned last_level,
-                                    unsigned first_layer,
-                                    unsigned last_layer)
+static bool noop_generate_mipmap(struct pipe_context *ctx,
+                                 struct pipe_resource *resource,
+                                 enum pipe_format format,
+                                 unsigned base_level,
+                                 unsigned last_level,
+                                 unsigned first_layer,
+                                 unsigned last_layer)
 {
    return true;
 }
@@ -312,6 +337,12 @@ static void noop_invalidate_resource(struct pipe_context *ctx,
 {
 }
 
+static void noop_set_context_param(struct pipe_context *ctx,
+                                   enum pipe_context_param param,
+                                   unsigned value)
+{
+}
+
 static struct pipe_context *noop_create_context(struct pipe_screen *screen,
                                                 void *priv, unsigned flags)
 {
@@ -351,6 +382,7 @@ static struct pipe_context *noop_create_context(struct pipe_screen *screen,
    ctx->buffer_subdata = noop_buffer_subdata;
    ctx->texture_subdata = noop_texture_subdata;
    ctx->invalidate_resource = noop_invalidate_resource;
+   ctx->set_context_param = noop_set_context_param;
    noop_init_state_functions(ctx);
 
    return ctx;
@@ -416,12 +448,12 @@ static int noop_get_compute_param(struct pipe_screen *pscreen,
    return screen->get_compute_param(screen, ir_type, param, ret);
 }
 
-static boolean noop_is_format_supported(struct pipe_screen* pscreen,
-                                        enum pipe_format format,
-                                        enum pipe_texture_target target,
-                                        unsigned sample_count,
-                                        unsigned storage_sample_count,
-                                        unsigned usage)
+static bool noop_is_format_supported(struct pipe_screen* pscreen,
+                                     enum pipe_format format,
+                                     enum pipe_texture_target target,
+                                     unsigned sample_count,
+                                     unsigned storage_sample_count,
+                                     unsigned usage)
 {
    struct pipe_screen *screen = ((struct noop_pipe_screen*)pscreen)->oscreen;
 
@@ -449,10 +481,10 @@ static void noop_fence_reference(struct pipe_screen *screen,
 {
 }
 
-static boolean noop_fence_finish(struct pipe_screen *screen,
-                                 struct pipe_context *ctx,
-                                 struct pipe_fence_handle *fence,
-                                 uint64_t timeout)
+static bool noop_fence_finish(struct pipe_screen *screen,
+                              struct pipe_context *ctx,
+                              struct pipe_fence_handle *fence,
+                              uint64_t timeout)
 {
    return true;
 }
@@ -466,6 +498,29 @@ static void noop_query_memory_info(struct pipe_screen *pscreen,
    screen->query_memory_info(screen, info);
 }
 
+static struct disk_cache *noop_get_disk_shader_cache(struct pipe_screen *pscreen)
+{
+   struct pipe_screen *screen = ((struct noop_pipe_screen*)pscreen)->oscreen;
+
+   return screen->get_disk_shader_cache(screen);
+}
+
+static const void *noop_get_compiler_options(struct pipe_screen *pscreen,
+                                             enum pipe_shader_ir ir,
+                                             enum pipe_shader_type shader)
+{
+   struct pipe_screen *screen = ((struct noop_pipe_screen*)pscreen)->oscreen;
+
+   return screen->get_compiler_options(screen, ir, shader);
+}
+
+static void noop_finalize_nir(struct pipe_screen *pscreen, void *nir, bool optimize)
+{
+   struct pipe_screen *screen = ((struct noop_pipe_screen*)pscreen)->oscreen;
+
+   screen->finalize_nir(screen, nir, optimize);
+}
+
 struct pipe_screen *noop_screen_create(struct pipe_screen *oscreen)
 {
    struct noop_pipe_screen *noop_screen;
@@ -495,12 +550,17 @@ struct pipe_screen *noop_screen_create(struct pipe_screen *oscreen)
    screen->resource_create = noop_resource_create;
    screen->resource_from_handle = noop_resource_from_handle;
    screen->resource_get_handle = noop_resource_get_handle;
+   if (oscreen->resource_get_param)
+      screen->resource_get_param = noop_resource_get_param;
    screen->resource_destroy = noop_resource_destroy;
    screen->flush_frontbuffer = noop_flush_frontbuffer;
    screen->get_timestamp = noop_get_timestamp;
    screen->fence_reference = noop_fence_reference;
    screen->fence_finish = noop_fence_finish;
    screen->query_memory_info = noop_query_memory_info;
+   screen->get_disk_shader_cache = noop_get_disk_shader_cache;
+   screen->get_compiler_options = noop_get_compiler_options;
+   screen->finalize_nir = noop_finalize_nir;
 
    return screen;
 }
