@@ -1,8 +1,8 @@
 /**************************************************************************
- * 
+ *
  * Copyright 2007 VMware, Inc.
  * All Rights Reserved.
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the
  * "Software"), to deal in the Software without restriction, including
@@ -10,11 +10,11 @@
  * distribute, sub license, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice (including the
  * next paragraph) shall be included in all copies or substantial portions
  * of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
@@ -22,7 +22,7 @@
  * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
  * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
  * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
  **************************************************************************/
 
 #ifndef U_INLINES_H
@@ -52,66 +52,67 @@ extern "C" {
 
 
 static inline void
-pipe_reference_init(struct pipe_reference *reference, unsigned count)
+pipe_reference_init(struct pipe_reference *dst, unsigned count)
 {
-   p_atomic_set(&reference->count, count);
+   p_atomic_set(&dst->count, count);
 }
 
 static inline boolean
-pipe_is_referenced(struct pipe_reference *reference)
+pipe_is_referenced(struct pipe_reference *src)
 {
-   return p_atomic_read(&reference->count) != 0;
+   return p_atomic_read(&src->count) != 0;
 }
 
 /**
  * Update reference counting.
  * The old thing pointed to, if any, will be unreferenced.
- * Both 'ptr' and 'reference' may be NULL.
+ * Both 'dst' and 'src' may be NULL.
  * \return TRUE if the object's refcount hits zero and should be destroyed.
  */
 static inline boolean
-pipe_reference_described(struct pipe_reference *ptr, 
-                         struct pipe_reference *reference, 
+pipe_reference_described(struct pipe_reference *dst,
+                         struct pipe_reference *src,
                          debug_reference_descriptor get_desc)
 {
-   boolean destroy = FALSE;
-
-   if(ptr != reference) {
-      /* bump the reference.count first */
-      if (reference) {
-         assert(pipe_is_referenced(reference));
-         p_atomic_inc(&reference->count);
-         debug_reference(reference, get_desc, 1);
+   if (dst != src) {
+      /* bump the src.count first */
+      if (src) {
+         ASSERTED int count = p_atomic_inc_return(&src->count);
+         assert(count != 1); /* src had to be referenced */
+         debug_reference(src, get_desc, 1);
       }
 
-      if (ptr) {
-         assert(pipe_is_referenced(ptr));
-         if (p_atomic_dec_zero(&ptr->count)) {
-            destroy = TRUE;
-         }
-         debug_reference(ptr, get_desc, -1);
+      if (dst) {
+         int count = p_atomic_dec_return(&dst->count);
+         assert(count != -1); /* dst had to be referenced */
+         debug_reference(dst, get_desc, -1);
+         if (!count)
+            return true;
       }
    }
 
-   return destroy;
+   return false;
 }
 
 static inline boolean
-pipe_reference(struct pipe_reference *ptr, struct pipe_reference *reference)
+pipe_reference(struct pipe_reference *dst, struct pipe_reference *src)
 {
-   return pipe_reference_described(ptr, reference, 
-                                   (debug_reference_descriptor)debug_describe_reference);
+   return pipe_reference_described(dst, src,
+                                   (debug_reference_descriptor)
+                                   debug_describe_reference);
 }
 
 static inline void
-pipe_surface_reference(struct pipe_surface **ptr, struct pipe_surface *surf)
+pipe_surface_reference(struct pipe_surface **dst, struct pipe_surface *src)
 {
-   struct pipe_surface *old_surf = *ptr;
+   struct pipe_surface *old_dst = *dst;
 
-   if (pipe_reference_described(&(*ptr)->reference, &surf->reference, 
-                                (debug_reference_descriptor)debug_describe_surface))
-      old_surf->context->surface_destroy(old_surf->context, old_surf);
-   *ptr = surf;
+   if (pipe_reference_described(old_dst ? &old_dst->reference : NULL,
+                                src ? &src->reference : NULL,
+                                (debug_reference_descriptor)
+                                debug_describe_surface))
+      old_dst->context->surface_destroy(old_dst->context, old_dst);
+   *dst = src;
 }
 
 /**
@@ -123,68 +124,108 @@ pipe_surface_reference(struct pipe_surface **ptr, struct pipe_surface *surf)
 static inline void
 pipe_surface_release(struct pipe_context *pipe, struct pipe_surface **ptr)
 {
-   if (pipe_reference_described(&(*ptr)->reference, NULL,
-                                (debug_reference_descriptor)debug_describe_surface))
-      pipe->surface_destroy(pipe, *ptr);
+   struct pipe_surface *old = *ptr;
+
+   if (pipe_reference_described(&old->reference, NULL,
+                                (debug_reference_descriptor)
+                                debug_describe_surface))
+      pipe->surface_destroy(pipe, old);
    *ptr = NULL;
 }
 
 
 static inline void
-pipe_resource_reference(struct pipe_resource **ptr, struct pipe_resource *tex)
+pipe_resource_reference(struct pipe_resource **dst, struct pipe_resource *src)
 {
-   struct pipe_resource *old_tex = *ptr;
+   struct pipe_resource *old_dst = *dst;
 
-   if (pipe_reference_described(&(*ptr)->reference, &tex->reference, 
-                                (debug_reference_descriptor)debug_describe_resource)) {
-      pipe_resource_reference(&old_tex->next, NULL);
-      old_tex->screen->resource_destroy(old_tex->screen, old_tex);
+   if (pipe_reference_described(old_dst ? &old_dst->reference : NULL,
+                                src ? &src->reference : NULL,
+                                (debug_reference_descriptor)
+                                debug_describe_resource)) {
+      /* Avoid recursion, which would prevent inlining this function */
+      do {
+         struct pipe_resource *next = old_dst->next;
+
+         old_dst->screen->resource_destroy(old_dst->screen, old_dst);
+         old_dst = next;
+      } while (pipe_reference_described(old_dst ? &old_dst->reference : NULL,
+                                        NULL,
+                                        (debug_reference_descriptor)
+                                        debug_describe_resource));
    }
-   *ptr = tex;
-}
-
-static inline void
-pipe_sampler_view_reference(struct pipe_sampler_view **ptr, struct pipe_sampler_view *view)
-{
-   struct pipe_sampler_view *old_view = *ptr;
-
-   if (pipe_reference_described(&(*ptr)->reference, &view->reference,
-                                (debug_reference_descriptor)debug_describe_sampler_view))
-      old_view->context->sampler_view_destroy(old_view->context, old_view);
-   *ptr = view;
+   *dst = src;
 }
 
 /**
- * Similar to pipe_sampler_view_reference() but always set the pointer to
- * NULL and pass in an explicit context.  Passing an explicit context is a
- * work-around for fixing a dangling context pointer problem when textures
- * are shared by multiple contexts.  XXX fix this someday.
+ * Same as pipe_surface_release, but used when pipe_context doesn't exist
+ * anymore.
  */
 static inline void
-pipe_sampler_view_release(struct pipe_context *ctx,
-                          struct pipe_sampler_view **ptr)
+pipe_surface_release_no_context(struct pipe_surface **ptr)
 {
-   struct pipe_sampler_view *old_view = *ptr;
-   if (*ptr && (*ptr)->context != ctx) {
-      debug_printf_once(("context mis-match in pipe_sampler_view_release()\n"));
-   }
-   if (pipe_reference_described(&(*ptr)->reference, NULL,
-                    (debug_reference_descriptor)debug_describe_sampler_view)) {
-      ctx->sampler_view_destroy(ctx, old_view);
+   struct pipe_surface *surf = *ptr;
+
+   if (pipe_reference_described(&surf->reference, NULL,
+                                (debug_reference_descriptor)
+                                debug_describe_surface)) {
+      /* trivially destroy pipe_surface */
+      pipe_resource_reference(&surf->texture, NULL);
+      free(surf);
    }
    *ptr = NULL;
 }
 
+/**
+ * Set *dst to \p src with proper reference counting.
+ *
+ * The caller must guarantee that \p src and *dst were created in
+ * the same context (if they exist), and that this must be the current context.
+ */
 static inline void
-pipe_so_target_reference(struct pipe_stream_output_target **ptr,
-                         struct pipe_stream_output_target *target)
+pipe_sampler_view_reference(struct pipe_sampler_view **dst,
+                            struct pipe_sampler_view *src)
 {
-   struct pipe_stream_output_target *old = *ptr;
+   struct pipe_sampler_view *old_dst = *dst;
 
-   if (pipe_reference_described(&(*ptr)->reference, &target->reference,
+   if (pipe_reference_described(old_dst ? &old_dst->reference : NULL,
+                                src ? &src->reference : NULL,
+                                (debug_reference_descriptor)
+                                debug_describe_sampler_view))
+      old_dst->context->sampler_view_destroy(old_dst->context, old_dst);
+   *dst = src;
+}
+
+static inline void
+pipe_so_target_reference(struct pipe_stream_output_target **dst,
+                         struct pipe_stream_output_target *src)
+{
+   struct pipe_stream_output_target *old_dst = *dst;
+
+   if (pipe_reference_described(old_dst ? &old_dst->reference : NULL,
+                     src ? &src->reference : NULL,
                      (debug_reference_descriptor)debug_describe_so_target))
-      old->context->stream_output_target_destroy(old->context, old);
-   *ptr = target;
+      old_dst->context->stream_output_target_destroy(old_dst->context, old_dst);
+   *dst = src;
+}
+
+static inline void
+pipe_vertex_buffer_unreference(struct pipe_vertex_buffer *dst)
+{
+   if (dst->is_user_buffer)
+      dst->buffer.user = NULL;
+   else
+      pipe_resource_reference(&dst->buffer.resource, NULL);
+}
+
+static inline void
+pipe_vertex_buffer_reference(struct pipe_vertex_buffer *dst,
+                             const struct pipe_vertex_buffer *src)
+{
+   pipe_vertex_buffer_unreference(dst);
+   if (!src->is_user_buffer)
+      pipe_resource_reference(&dst->buffer.resource, src->buffer.resource);
+   memcpy(dst, src, sizeof(*src));
 }
 
 static inline void
@@ -235,10 +276,10 @@ pipe_surface_equal(struct pipe_surface *s1, struct pipe_surface *s2)
  * \param usage  a PIPE_USAGE_x value
  */
 static inline struct pipe_resource *
-pipe_buffer_create( struct pipe_screen *screen,
-		    unsigned bind,
-		    enum pipe_resource_usage usage,
-		    unsigned size )
+pipe_buffer_create(struct pipe_screen *screen,
+                   unsigned bind,
+                   enum pipe_resource_usage usage,
+                   unsigned size)
 {
    struct pipe_resource buffer;
    memset(&buffer, 0, sizeof buffer);
@@ -255,20 +296,41 @@ pipe_buffer_create( struct pipe_screen *screen,
 }
 
 
+static inline struct pipe_resource *
+pipe_buffer_create_const0(struct pipe_screen *screen,
+                          unsigned bind,
+                          enum pipe_resource_usage usage,
+                          unsigned size)
+{
+   struct pipe_resource buffer;
+   memset(&buffer, 0, sizeof buffer);
+   buffer.target = PIPE_BUFFER;
+   buffer.format = PIPE_FORMAT_R8_UNORM;
+   buffer.bind = bind;
+   buffer.usage = usage;
+   buffer.flags = screen->get_param(screen, PIPE_CAP_CONSTBUF0_FLAGS);
+   buffer.width0 = size;
+   buffer.height0 = 1;
+   buffer.depth0 = 1;
+   buffer.array_size = 1;
+   return screen->resource_create(screen, &buffer);
+}
+
+
 /**
  * Map a range of a resource.
- * \param offset  start of region, in bytes 
- * \param length  size of region, in bytes 
+ * \param offset  start of region, in bytes
+ * \param length  size of region, in bytes
  * \param access  bitmask of PIPE_TRANSFER_x flags
  * \param transfer  returns a transfer object
  */
 static inline void *
 pipe_buffer_map_range(struct pipe_context *pipe,
-		      struct pipe_resource *buffer,
-		      unsigned offset,
-		      unsigned length,
-		      unsigned access,
-		      struct pipe_transfer **transfer)
+                      struct pipe_resource *buffer,
+                      unsigned offset,
+                      unsigned length,
+                      unsigned access,
+                      struct pipe_transfer **transfer)
 {
    struct pipe_box box;
    void *map;
@@ -299,7 +361,8 @@ pipe_buffer_map(struct pipe_context *pipe,
                 unsigned access,
                 struct pipe_transfer **transfer)
 {
-   return pipe_buffer_map_range(pipe, buffer, 0, buffer->width0, access, transfer);
+   return pipe_buffer_map_range(pipe, buffer, 0, buffer->width0,
+                                access, transfer);
 }
 
 
@@ -393,10 +456,10 @@ pipe_buffer_read(struct pipe_context *pipe,
    ubyte *map;
 
    map = (ubyte *) pipe_buffer_map_range(pipe,
-					 buf,
-					 offset, size,
-					 PIPE_TRANSFER_READ,
-					 &src_transfer);
+                                         buf,
+                                         offset, size,
+                                         PIPE_TRANSFER_READ,
+                                         &src_transfer);
    if (!map)
       return;
 
@@ -451,14 +514,15 @@ pipe_transfer_map_3d(struct pipe_context *context,
 }
 
 static inline void
-pipe_transfer_unmap( struct pipe_context *context,
-                     struct pipe_transfer *transfer )
+pipe_transfer_unmap(struct pipe_context *context,
+                    struct pipe_transfer *transfer)
 {
-   context->transfer_unmap( context, transfer );
+   context->transfer_unmap(context, transfer);
 }
 
 static inline void
-pipe_set_constant_buffer(struct pipe_context *pipe, uint shader, uint index,
+pipe_set_constant_buffer(struct pipe_context *pipe,
+                         enum pipe_shader_type shader, uint index,
                          struct pipe_resource *buf)
 {
    if (buf) {
@@ -510,7 +574,9 @@ util_query_clear_result(union pipe_query_result *result, unsigned type)
 {
    switch (type) {
    case PIPE_QUERY_OCCLUSION_PREDICATE:
+   case PIPE_QUERY_OCCLUSION_PREDICATE_CONSERVATIVE:
    case PIPE_QUERY_SO_OVERFLOW_PREDICATE:
+   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
    case PIPE_QUERY_GPU_FINISHED:
       result->b = FALSE;
       break;
@@ -536,7 +602,7 @@ util_query_clear_result(union pipe_query_result *result, unsigned type)
 }
 
 /** Convert PIPE_TEXTURE_x to TGSI_TEXTURE_x */
-static inline unsigned
+static inline enum tgsi_texture_type
 util_pipe_tex_to_tgsi_tex(enum pipe_texture_target pipe_tex_target,
                           unsigned nr_samples)
 {
@@ -599,6 +665,22 @@ util_copy_constant_buffer(struct pipe_constant_buffer *dst,
    }
 }
 
+static inline void
+util_copy_shader_buffer(struct pipe_shader_buffer *dst,
+                        const struct pipe_shader_buffer *src)
+{
+   if (src) {
+      pipe_resource_reference(&dst->buffer, src->buffer);
+      dst->buffer_offset = src->buffer_offset;
+      dst->buffer_size = src->buffer_size;
+   }
+   else {
+      pipe_resource_reference(&dst->buffer, NULL);
+      dst->buffer_offset = 0;
+      dst->buffer_size = 0;
+   }
+}
+
 static inline void
 util_copy_image_view(struct pipe_image_view *dst,
                      const struct pipe_image_view *src)
@@ -607,11 +689,13 @@ util_copy_image_view(struct pipe_image_view *dst,
       pipe_resource_reference(&dst->resource, src->resource);
       dst->format = src->format;
       dst->access = src->access;
+      dst->shader_access = src->shader_access;
       dst->u = src->u;
    } else {
       pipe_resource_reference(&dst->resource, NULL);
       dst->format = PIPE_FORMAT_NONE;
       dst->access = 0;
+      dst->shader_access = 0;
       memset(&dst->u, 0, sizeof(dst->u));
    }
 }
@@ -634,6 +718,12 @@ util_max_layer(const struct pipe_resource *r, unsigned level)
    }
 }
 
+static inline unsigned
+util_num_layers(const struct pipe_resource *r, unsigned level)
+{
+   return util_max_layer(r, level) + 1;
+}
+
 static inline bool
 util_texrange_covers_whole_level(const struct pipe_resource *tex,
                                  unsigned level, unsigned x, unsigned y,
@@ -643,7 +733,18 @@ util_texrange_covers_whole_level(const struct pipe_resource *tex,
    return x == 0 && y == 0 && z == 0 &&
           width == u_minify(tex->width0, level) &&
           height == u_minify(tex->height0, level) &&
-          depth == util_max_layer(tex, level) + 1;
+          depth == util_num_layers(tex, level);
+}
+
+static inline struct pipe_context *
+pipe_create_multimedia_context(struct pipe_screen *screen)
+{
+   unsigned flags = 0;
+
+   if (!screen->get_param(screen, PIPE_CAP_GRAPHICS))
+      flags |= PIPE_CONTEXT_COMPUTE_ONLY;
+
+   return screen->context_create(screen, NULL, flags);
 }
 
 #ifdef __cplusplus
