@@ -24,9 +24,9 @@
 #include "pipe/p_screen.h"
 
 #include "util/u_box.h"
-#include "util/u_format.h"
-#include "util/u_format_rgtc.h"
-#include "util/u_format_zs.h"
+#include "util/format/u_format.h"
+#include "util/format/u_format_rgtc.h"
+#include "util/format/u_format_zs.h"
 #include "util/u_inlines.h"
 #include "util/u_transfer_helper.h"
 
@@ -300,32 +300,33 @@ u_transfer_helper_transfer_map(struct pipe_context *pctx,
             unreachable("Unexpected format");
          }
       }
-   } else if (needs_pack(usage) &&
-              util_format_description(prsc->format)->layout == UTIL_FORMAT_LAYOUT_RGTC) {
-      switch (prsc->format) {
-      case PIPE_FORMAT_RGTC1_UNORM:
-      case PIPE_FORMAT_RGTC1_SNORM:
-      case PIPE_FORMAT_LATC1_UNORM:
-      case PIPE_FORMAT_LATC1_SNORM:
-         util_format_rgtc1_unorm_pack_rgba_8unorm(trans->staging,
-                                                  ptrans->stride,
-                                                  trans->ptr,
-                                                  trans->trans->stride,
-                                                  width, height);
-         break;
-      case PIPE_FORMAT_RGTC2_UNORM:
-      case PIPE_FORMAT_RGTC2_SNORM:
-      case PIPE_FORMAT_LATC2_UNORM:
-      case PIPE_FORMAT_LATC2_SNORM:
-         util_format_rgtc2_unorm_pack_rgba_8unorm(trans->staging,
-                                                  ptrans->stride,
-                                                  trans->ptr,
-                                                  trans->trans->stride,
-                                                  width, height);
-         break;
-      default:
-         assert(!"Unexpected format");
-         break;
+   } else if (util_format_description(prsc->format)->layout == UTIL_FORMAT_LAYOUT_RGTC) {
+      if (needs_pack(usage)) {
+         switch (prsc->format) {
+         case PIPE_FORMAT_RGTC1_UNORM:
+         case PIPE_FORMAT_RGTC1_SNORM:
+         case PIPE_FORMAT_LATC1_UNORM:
+         case PIPE_FORMAT_LATC1_SNORM:
+            util_format_rgtc1_unorm_pack_rgba_8unorm(trans->staging,
+                                                     ptrans->stride,
+                                                     trans->ptr,
+                                                     trans->trans->stride,
+                                                     width, height);
+            break;
+         case PIPE_FORMAT_RGTC2_UNORM:
+         case PIPE_FORMAT_RGTC2_SNORM:
+         case PIPE_FORMAT_LATC2_UNORM:
+         case PIPE_FORMAT_LATC2_SNORM:
+            util_format_rgtc2_unorm_pack_rgba_8unorm(trans->staging,
+                                                     ptrans->stride,
+                                                     trans->ptr,
+                                                     trans->trans->stride,
+                                                     width, height);
+            break;
+         default:
+            assert(!"Unexpected format");
+            break;
+         }
       }
    } else {
       unreachable("bleh");
@@ -511,6 +512,9 @@ u_transfer_helper_transfer_unmap(struct pipe_context *pctx,
             helper->vtbl->transfer_unmap(pctx, trans->trans2);
       }
 
+      pipe_resource_reference(&ptrans->resource, NULL);
+
+      free(trans->staging);
       free(trans);
    } else {
       helper->vtbl->transfer_unmap(pctx, ptrans);
