@@ -29,7 +29,7 @@
 #include "util/u_string.h"
 #include "util/u_memory.h"
 #include "util/u_inlines.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/bitset.h"
 
 #include "freedreno_program.h"
@@ -39,52 +39,50 @@
 #include "fd6_texture.h"
 #include "fd6_format.h"
 
-static struct ir3_shader *
-create_shader_stateobj(struct pipe_context *pctx, const struct pipe_shader_state *cso,
-		gl_shader_stage type)
-{
-	struct fd_context *ctx = fd_context(pctx);
-	struct ir3_compiler *compiler = ctx->screen->compiler;
-	return ir3_shader_create(compiler, cso, type, &ctx->debug, pctx->screen);
-}
-
-static void *
-fd6_fp_state_create(struct pipe_context *pctx,
-		const struct pipe_shader_state *cso)
-{
-	return create_shader_stateobj(pctx, cso, MESA_SHADER_FRAGMENT);
-}
-
-static void
-fd6_fp_state_delete(struct pipe_context *pctx, void *hwcso)
-{
-	struct ir3_shader *so = hwcso;
-	struct fd_context *ctx = fd_context(pctx);
-	ir3_cache_invalidate(fd6_context(ctx)->shader_cache, hwcso);
-	ir3_shader_destroy(so);
-}
-
-static void *
-fd6_vp_state_create(struct pipe_context *pctx,
-		const struct pipe_shader_state *cso)
-{
-	return create_shader_stateobj(pctx, cso, MESA_SHADER_VERTEX);
-}
-
-static void
-fd6_vp_state_delete(struct pipe_context *pctx, void *hwcso)
-{
-	struct ir3_shader *so = hwcso;
-	struct fd_context *ctx = fd_context(pctx);
-	ir3_cache_invalidate(fd6_context(ctx)->shader_cache, hwcso);
-	ir3_shader_destroy(so);
-}
-
 void
 fd6_emit_shader(struct fd_ringbuffer *ring, const struct ir3_shader_variant *so)
 {
 	enum a6xx_state_block sb = fd6_stage2shadersb(so->type);
 
+	uint32_t obj_start;
+	uint32_t instrlen;
+
+	switch (so->type) {
+	case MESA_SHADER_VERTEX:
+		obj_start = REG_A6XX_SP_VS_OBJ_START_LO;
+		instrlen = REG_A6XX_SP_VS_INSTRLEN;
+		break;
+	case MESA_SHADER_TESS_CTRL:
+		obj_start = REG_A6XX_SP_HS_OBJ_START_LO;
+		instrlen = REG_A6XX_SP_HS_INSTRLEN;
+		break;
+	case MESA_SHADER_TESS_EVAL:
+		obj_start = REG_A6XX_SP_DS_OBJ_START_LO;
+		instrlen = REG_A6XX_SP_DS_INSTRLEN;
+		break;
+	case MESA_SHADER_GEOMETRY:
+		obj_start = REG_A6XX_SP_GS_OBJ_START_LO;
+		instrlen = REG_A6XX_SP_GS_INSTRLEN;
+		break;
+	case MESA_SHADER_FRAGMENT:
+		obj_start = REG_A6XX_SP_FS_OBJ_START_LO;
+		instrlen = REG_A6XX_SP_FS_INSTRLEN;
+		break;
+	case MESA_SHADER_COMPUTE:
+	case MESA_SHADER_KERNEL:
+		obj_start = REG_A6XX_SP_CS_OBJ_START_LO;
+		instrlen = REG_A6XX_SP_CS_INSTRLEN;
+		break;
+	case MESA_SHADER_NONE:
+		unreachable("");
+	}
+
+	OUT_PKT4(ring, instrlen, 1);
+	OUT_RING(ring, so->instrlen);
+
+	OUT_PKT4(ring, obj_start, 2);
+	OUT_RELOC(ring, so->bo, 0, 0, 0);
+
 	OUT_PKT7(ring, fd6_stage2opcode(so->type), 3);
 	OUT_RING(ring, CP_LOAD_STATE6_0_DST_OFF(0) |
 			CP_LOAD_STATE6_0_STATE_TYPE(ST6_SHADER) |
@@ -200,47 +198,64 @@ setup_config_stateobj(struct fd_ringbuffer *ring, struct fd6_program_state *stat
 	OUT_PKT4(ring, REG_A6XX_HLSQ_UPDATE_CNTL, 1);
 	OUT_RING(ring, 0xff);        /* XXX */
 
-	debug_assert(state->vs->constlen >= state->bs->constlen);
+	if (state->ds)
+		debug_assert(state->ds->constlen >= state->bs->constlen);
+	else
+		debug_assert(state->vs->constlen >= state->bs->constlen);
 
 	OUT_PKT4(ring, REG_A6XX_HLSQ_VS_CNTL, 4);
 	OUT_RING(ring, A6XX_HLSQ_VS_CNTL_CONSTLEN(align(state->vs->constlen, 4)) |
 			A6XX_HLSQ_VS_CNTL_ENABLED);
-	OUT_RING(ring, A6XX_HLSQ_HS_CNTL_CONSTLEN(0));
-	OUT_RING(ring, A6XX_HLSQ_DS_CNTL_CONSTLEN(0));
-	OUT_RING(ring, A6XX_HLSQ_GS_CNTL_CONSTLEN(0));
-
+	OUT_RING(ring, COND(state->hs,
+					A6XX_HLSQ_HS_CNTL_ENABLED |
+					A6XX_HLSQ_HS_CNTL_CONSTLEN(align(state->hs->constlen, 4))));
+	OUT_RING(ring, COND(state->ds,
+					A6XX_HLSQ_DS_CNTL_ENABLED |
+					A6XX_HLSQ_DS_CNTL_CONSTLEN(align(state->ds->constlen, 4))));
+	OUT_RING(ring, COND(state->gs,
+					A6XX_HLSQ_GS_CNTL_ENABLED |
+					A6XX_HLSQ_GS_CNTL_CONSTLEN(align(state->gs->constlen, 4))));
 	OUT_PKT4(ring, REG_A6XX_HLSQ_FS_CNTL, 1);
 	OUT_RING(ring, A6XX_HLSQ_FS_CNTL_CONSTLEN(align(state->fs->constlen, 4)) |
 			A6XX_HLSQ_FS_CNTL_ENABLED);
 
 	OUT_PKT4(ring, REG_A6XX_SP_VS_CONFIG, 1);
 	OUT_RING(ring, COND(state->vs, A6XX_SP_VS_CONFIG_ENABLED) |
-			A6XX_SP_VS_CONFIG_NIBO(state->vs->image_mapping.num_ibo) |
+			A6XX_SP_VS_CONFIG_NIBO(ir3_shader_nibo(state->vs)) |
 			A6XX_SP_VS_CONFIG_NTEX(state->vs->num_samp) |
 			A6XX_SP_VS_CONFIG_NSAMP(state->vs->num_samp));
 
+	OUT_PKT4(ring, REG_A6XX_SP_HS_CONFIG, 1);
+	OUT_RING(ring, COND(state->hs,
+					A6XX_SP_HS_CONFIG_ENABLED |
+					A6XX_SP_HS_CONFIG_NIBO(ir3_shader_nibo(state->hs)) |
+					A6XX_SP_HS_CONFIG_NTEX(state->hs->num_samp) |
+					A6XX_SP_HS_CONFIG_NSAMP(state->hs->num_samp)));
+
+	OUT_PKT4(ring, REG_A6XX_SP_DS_CONFIG, 1);
+	OUT_RING(ring, COND(state->ds,
+					A6XX_SP_DS_CONFIG_ENABLED |
+					A6XX_SP_DS_CONFIG_NIBO(ir3_shader_nibo(state->ds)) |
+					A6XX_SP_DS_CONFIG_NTEX(state->ds->num_samp) |
+					A6XX_SP_DS_CONFIG_NSAMP(state->ds->num_samp)));
+
+	OUT_PKT4(ring, REG_A6XX_SP_GS_CONFIG, 1);
+	OUT_RING(ring, COND(state->gs,
+					A6XX_SP_GS_CONFIG_ENABLED |
+					A6XX_SP_GS_CONFIG_NIBO(ir3_shader_nibo(state->gs)) |
+					A6XX_SP_GS_CONFIG_NTEX(state->gs->num_samp) |
+					A6XX_SP_GS_CONFIG_NSAMP(state->gs->num_samp)));
+
 	OUT_PKT4(ring, REG_A6XX_SP_FS_CONFIG, 1);
 	OUT_RING(ring, COND(state->fs, A6XX_SP_FS_CONFIG_ENABLED) |
-			A6XX_SP_FS_CONFIG_NIBO(state->fs->image_mapping.num_ibo) |
+			A6XX_SP_FS_CONFIG_NIBO(ir3_shader_nibo(state->fs)) |
 			A6XX_SP_FS_CONFIG_NTEX(state->fs->num_samp) |
 			A6XX_SP_FS_CONFIG_NSAMP(state->fs->num_samp));
 
-	OUT_PKT4(ring, REG_A6XX_SP_HS_CONFIG, 1);
-	OUT_RING(ring, COND(false, A6XX_SP_HS_CONFIG_ENABLED));
-
-	OUT_PKT4(ring, REG_A6XX_SP_DS_CONFIG, 1);
-	OUT_RING(ring, COND(false, A6XX_SP_DS_CONFIG_ENABLED));
-
-	OUT_PKT4(ring, REG_A6XX_SP_GS_CONFIG, 1);
-	OUT_RING(ring, COND(false, A6XX_SP_GS_CONFIG_ENABLED));
-
 	OUT_PKT4(ring, REG_A6XX_SP_IBO_COUNT, 1);
-	OUT_RING(ring, state->fs->image_mapping.num_ibo);
+	OUT_RING(ring, ir3_shader_nibo(state->fs));
 }
 
-#define VALIDREG(r)      ((r) != regid(63,0))
-#define CONDREG(r, val)  COND(VALIDREG(r), (val))
-
 static inline uint32_t
 next_regid(uint32_t reg, uint32_t increment)
 {
@@ -258,16 +273,28 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	uint32_t pos_regid, psize_regid, color_regid[8], posz_regid;
 	uint32_t face_regid, coord_regid, zwcoord_regid, samp_id_regid;
 	uint32_t smask_in_regid, smask_regid;
-	uint32_t vertex_regid, instance_regid;
+	uint32_t vertex_regid, instance_regid, layer_regid, primitive_regid;
+	uint32_t hs_invocation_regid;
+	uint32_t tess_coord_x_regid, tess_coord_y_regid, hs_patch_regid, ds_patch_regid;
 	uint32_t ij_pix_regid, ij_samp_regid, ij_cent_regid, ij_size_regid;
+	uint32_t gs_header_regid;
 	enum a3xx_threadsize fssz;
-	uint8_t psize_loc = ~0;
+	uint8_t psize_loc = ~0, pos_loc = ~0, layer_loc = ~0;
 	int i, j;
 
 	static const struct ir3_shader_variant dummy_fs = {0};
 	const struct ir3_shader_variant *vs = binning_pass ? state->bs : state->vs;
+	const struct ir3_shader_variant *hs = state->hs;
+	const struct ir3_shader_variant *ds = state->ds;
+	const struct ir3_shader_variant *gs = state->gs;
 	const struct ir3_shader_variant *fs = binning_pass ? &dummy_fs : state->fs;
 
+	/* binning VS is wrong when GS is present, so use nonbinning VS
+	 * TODO: compile both binning VS/GS variants correctly
+	 */
+	if (binning_pass && state->gs)
+		vs = state->vs;
+
 	bool sample_shading = fs->per_samp | key->sample_shading;
 
 	fssz = FOUR_QUADS;
@@ -277,6 +304,35 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	vertex_regid = ir3_find_sysval_regid(vs, SYSTEM_VALUE_VERTEX_ID);
 	instance_regid = ir3_find_sysval_regid(vs, SYSTEM_VALUE_INSTANCE_ID);
 
+	if (hs) {
+		tess_coord_x_regid = ir3_find_sysval_regid(ds, SYSTEM_VALUE_TESS_COORD);
+		tess_coord_y_regid = next_regid(tess_coord_x_regid, 1);
+		hs_patch_regid = ir3_find_sysval_regid(hs, SYSTEM_VALUE_PRIMITIVE_ID);
+		ds_patch_regid = ir3_find_sysval_regid(ds, SYSTEM_VALUE_PRIMITIVE_ID);
+		hs_invocation_regid = ir3_find_sysval_regid(hs, SYSTEM_VALUE_TCS_HEADER_IR3);
+
+		pos_regid = ir3_find_output_regid(ds, VARYING_SLOT_POS);
+		psize_regid = ir3_find_output_regid(ds, VARYING_SLOT_PSIZ);
+	} else {
+		tess_coord_x_regid = regid(63, 0);
+		tess_coord_y_regid = regid(63, 0);
+		hs_patch_regid = regid(63, 0);
+		ds_patch_regid = regid(63, 0);
+		hs_invocation_regid = regid(63, 0);
+	}
+
+	if (gs) {
+		gs_header_regid = ir3_find_sysval_regid(gs, SYSTEM_VALUE_GS_HEADER_IR3);
+		primitive_regid = ir3_find_sysval_regid(gs, SYSTEM_VALUE_PRIMITIVE_ID);
+		pos_regid = ir3_find_output_regid(gs, VARYING_SLOT_POS);
+		psize_regid = ir3_find_output_regid(gs, VARYING_SLOT_PSIZ);
+		layer_regid = ir3_find_output_regid(gs, VARYING_SLOT_LAYER);
+	} else {
+		gs_header_regid = regid(63, 0);
+		primitive_regid = regid(63, 0);
+		layer_regid = regid(63, 0);
+	}
+
 	if (fs->color0_mrt) {
 		color_regid[0] = color_regid[1] = color_regid[2] = color_regid[3] =
 		color_regid[4] = color_regid[5] = color_regid[6] = color_regid[7] =
@@ -297,13 +353,22 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	face_regid      = ir3_find_sysval_regid(fs, SYSTEM_VALUE_FRONT_FACE);
 	coord_regid     = ir3_find_sysval_regid(fs, SYSTEM_VALUE_FRAG_COORD);
 	zwcoord_regid   = next_regid(coord_regid, 2);
-	ij_pix_regid    = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_PIXEL);
-	ij_samp_regid   = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_SAMPLE);
-	ij_cent_regid   = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_CENTROID);
-	ij_size_regid   = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_SIZE);
+	ij_pix_regid    = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_PERSP_PIXEL);
+	ij_samp_regid   = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_PERSP_SAMPLE);
+	ij_cent_regid   = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_PERSP_CENTROID);
+	ij_size_regid   = ir3_find_sysval_regid(fs, SYSTEM_VALUE_BARYCENTRIC_PERSP_SIZE);
 	posz_regid      = ir3_find_output_regid(fs, FRAG_RESULT_DEPTH);
 	smask_regid     = ir3_find_output_regid(fs, FRAG_RESULT_SAMPLE_MASK);
 
+	/* If we have pre-dispatch texture fetches, then ij_pix should not
+	 * be DCE'd, even if not actually used in the shader itself:
+	 */
+	if (fs->num_sampler_prefetch > 0) {
+		assert(VALIDREG(ij_pix_regid));
+		/* also, it seems like ij_pix is *required* to be r0.x */
+		assert(ij_pix_regid == regid(0, 0));
+	}
+
 	/* we can't write gl_SampleMask for !msaa..  if b0 is zero then we
 	 * end up masking the single sample!!
 	 */
@@ -314,56 +379,58 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	 * emitted if frag-prog is dirty vs if vert-prog is dirty..
 	 */
 
-	OUT_PKT4(ring, REG_A6XX_SP_VS_INSTRLEN, 1);
-	OUT_RING(ring, vs->instrlen);							/* SP_VS_INSTRLEN */
-
-	OUT_PKT4(ring, REG_A6XX_SP_HS_UNKNOWN_A831, 1);
-	OUT_RING(ring, 0);
-
-	OUT_PKT4(ring, REG_A6XX_SP_HS_INSTRLEN, 1);
-	OUT_RING(ring, 0);										/* SP_HS_INSTRLEN */
-
-	OUT_PKT4(ring, REG_A6XX_SP_DS_INSTRLEN, 1);
-	OUT_RING(ring, 0);										/* SP_DS_INSTRLEN */
-
-	OUT_PKT4(ring, REG_A6XX_SP_GS_UNKNOWN_A871, 1);
-	OUT_RING(ring, 0);
-
-	OUT_PKT4(ring, REG_A6XX_SP_GS_INSTRLEN, 1);
-	OUT_RING(ring, 0);										/* SP_GS_INSTRLEN */
-
-	/* I believe this is related to pre-dispatch texture fetch.. we probably
-	 * should't turn it on by accident:
-	 */
-	OUT_PKT4(ring, REG_A6XX_SP_UNKNOWN_A99E, 1);
+	OUT_PKT4(ring, REG_A6XX_SP_HS_UNKNOWN_A833, 1);
 	OUT_RING(ring, 0x0);
 
+	OUT_PKT4(ring, REG_A6XX_SP_FS_PREFETCH_CNTL, 1 + fs->num_sampler_prefetch);
+	OUT_RING(ring, A6XX_SP_FS_PREFETCH_CNTL_COUNT(fs->num_sampler_prefetch) |
+			A6XX_SP_FS_PREFETCH_CNTL_UNK4(regid(63, 0)) |
+			0x7000);    // XXX
+	for (int i = 0; i < fs->num_sampler_prefetch; i++) {
+		const struct ir3_sampler_prefetch *prefetch = &fs->sampler_prefetch[i];
+		OUT_RING(ring, A6XX_SP_FS_PREFETCH_CMD_SRC(prefetch->src) |
+				A6XX_SP_FS_PREFETCH_CMD_SAMP_ID(prefetch->samp_id) |
+				A6XX_SP_FS_PREFETCH_CMD_TEX_ID(prefetch->tex_id) |
+				A6XX_SP_FS_PREFETCH_CMD_DST(prefetch->dst) |
+				A6XX_SP_FS_PREFETCH_CMD_WRMASK(prefetch->wrmask) |
+				COND(prefetch->half_precision, A6XX_SP_FS_PREFETCH_CMD_HALF) |
+				A6XX_SP_FS_PREFETCH_CMD_CMD(prefetch->cmd));
+	}
+
 	OUT_PKT4(ring, REG_A6XX_SP_UNKNOWN_A9A8, 1);
 	OUT_RING(ring, 0);
 
 	OUT_PKT4(ring, REG_A6XX_SP_UNKNOWN_AB00, 1);
 	OUT_RING(ring, 0x5);
 
-	OUT_PKT4(ring, REG_A6XX_SP_FS_INSTRLEN, 1);
-	OUT_RING(ring, fs->instrlen);							  /* SP_FS_INSTRLEN */
-
 	OUT_PKT4(ring, REG_A6XX_SP_FS_OUTPUT_CNTL0, 1);
 	OUT_RING(ring, A6XX_SP_FS_OUTPUT_CNTL0_DEPTH_REGID(posz_regid) |
 			 A6XX_SP_FS_OUTPUT_CNTL0_SAMPMASK_REGID(smask_regid) |
 			 0xfc000000);
 
+	enum a3xx_threadsize vssz;
+	uint32_t vsregs;
+	if (ds || hs) {
+		vssz = TWO_QUADS;
+		vsregs = 0;
+	} else {
+		vssz = FOUR_QUADS;
+		vsregs = A6XX_SP_VS_CTRL_REG0_MERGEDREGS;
+	}
+
 	OUT_PKT4(ring, REG_A6XX_SP_VS_CTRL_REG0, 1);
-	OUT_RING(ring, A6XX_SP_VS_CTRL_REG0_THREADSIZE(fssz) |
+	OUT_RING(ring, A6XX_SP_VS_CTRL_REG0_THREADSIZE(vssz) |
 			A6XX_SP_VS_CTRL_REG0_FULLREGFOOTPRINT(vs->info.max_reg + 1) |
-			A6XX_SP_VS_CTRL_REG0_MERGEDREGS |
+			vsregs |
 			A6XX_SP_VS_CTRL_REG0_BRANCHSTACK(vs->branchstack) |
 			COND(vs->need_pixlod, A6XX_SP_VS_CTRL_REG0_PIXLODENABLE));
 
-	struct ir3_shader_linkage l = {0};
-	ir3_link_shaders(&l, vs, fs);
+	fd6_emit_shader(ring, vs);
+	ir3_emit_immediates(screen, vs, ring);
 
-	if ((vs->shader->stream_output.num_outputs > 0) && !binning_pass)
-		link_stream_out(&l, vs);
+	struct ir3_shader_linkage l = {0};
+	const struct ir3_shader_variant *last_shader = fd6_last_shader(state);
+	ir3_link_shaders(&l, last_shader, fs);
 
 	BITSET_DECLARE(varbs, 128) = {0};
 	uint32_t *varmask = (uint32_t *)varbs;
@@ -378,23 +445,39 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	OUT_RING(ring, ~varmask[2]);  /* VPC_VAR[2].DISABLE */
 	OUT_RING(ring, ~varmask[3]);  /* VPC_VAR[3].DISABLE */
 
-	/* a6xx appends pos/psize to end of the linkage map: */
-	if (VALIDREG(pos_regid))
+	/* Add stream out outputs after computing the VPC_VAR_DISABLE bitmask. */
+	if (last_shader->shader->stream_output.num_outputs > 0)
+		link_stream_out(&l, last_shader);
+
+	if (VALIDREG(layer_regid)) {
+		layer_loc = l.max_loc;
+		ir3_link_add(&l, layer_regid, 0x1, l.max_loc);
+	}
+
+	if (VALIDREG(pos_regid)) {
+		pos_loc = l.max_loc;
 		ir3_link_add(&l, pos_regid, 0xf, l.max_loc);
+	}
 
 	if (VALIDREG(psize_regid)) {
 		psize_loc = l.max_loc;
 		ir3_link_add(&l, psize_regid, 0x1, l.max_loc);
 	}
 
-	if ((vs->shader->stream_output.num_outputs > 0) && !binning_pass) {
-		setup_stream_out(state, vs, &l);
+	if (last_shader->shader->stream_output.num_outputs > 0) {
+		setup_stream_out(state, last_shader, &l);
 	}
 
-	for (i = 0, j = 0; (i < 16) && (j < l.cnt); i++) {
-		uint32_t reg = 0;
+	debug_assert(l.cnt < 32);
+	if (gs)
+		OUT_PKT4(ring, REG_A6XX_SP_GS_OUT_REG(0), DIV_ROUND_UP(l.cnt, 2));
+	else if (ds)
+		OUT_PKT4(ring, REG_A6XX_SP_DS_OUT_REG(0), DIV_ROUND_UP(l.cnt, 2));
+	else
+		OUT_PKT4(ring, REG_A6XX_SP_VS_OUT_REG(0), DIV_ROUND_UP(l.cnt, 2));
 
-		OUT_PKT4(ring, REG_A6XX_SP_VS_OUT_REG(i), 1);
+	for (j = 0; j < l.cnt; ) {
+		uint32_t reg = 0;
 
 		reg |= A6XX_SP_VS_OUT_REG_A_REGID(l.var[j].regid);
 		reg |= A6XX_SP_VS_OUT_REG_A_COMPMASK(l.var[j].compmask);
@@ -407,10 +490,15 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 		OUT_RING(ring, reg);
 	}
 
-	for (i = 0, j = 0; (i < 8) && (j < l.cnt); i++) {
-		uint32_t reg = 0;
+	if (gs)
+		OUT_PKT4(ring, REG_A6XX_SP_GS_VPC_DST_REG(0), DIV_ROUND_UP(l.cnt, 4));
+	else if (ds)
+		OUT_PKT4(ring, REG_A6XX_SP_DS_VPC_DST_REG(0), DIV_ROUND_UP(l.cnt, 4));
+	else
+		OUT_PKT4(ring, REG_A6XX_SP_VS_VPC_DST_REG(0), DIV_ROUND_UP(l.cnt, 4));
 
-		OUT_PKT4(ring, REG_A6XX_SP_VS_VPC_DST_REG(i), 1);
+	for (j = 0; j < l.cnt; ) {
+		uint32_t reg = 0;
 
 		reg |= A6XX_SP_VS_VPC_DST_REG_OUTLOC0(l.var[j++].loc);
 		reg |= A6XX_SP_VS_VPC_DST_REG_OUTLOC1(l.var[j++].loc);
@@ -420,12 +508,87 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 		OUT_RING(ring, reg);
 	}
 
-	OUT_PKT4(ring, REG_A6XX_SP_VS_OBJ_START_LO, 2);
-	OUT_RELOC(ring, vs->bo, 0, 0, 0);  /* SP_VS_OBJ_START_LO/HI */
+	if (hs) {
+		OUT_PKT4(ring, REG_A6XX_SP_HS_CTRL_REG0, 1);
+		OUT_RING(ring, A6XX_SP_HS_CTRL_REG0_THREADSIZE(TWO_QUADS) |
+			A6XX_SP_HS_CTRL_REG0_FULLREGFOOTPRINT(hs->info.max_reg + 1) |
+			A6XX_SP_HS_CTRL_REG0_BRANCHSTACK(hs->branchstack) |
+			COND(hs->need_pixlod, A6XX_SP_HS_CTRL_REG0_PIXLODENABLE));
 
-	if (vs->instrlen)
-		fd6_emit_shader(ring, vs);
+		fd6_emit_shader(ring, hs);
+		ir3_emit_immediates(screen, hs, ring);
+		ir3_emit_link_map(screen, vs, hs, ring);
 
+		OUT_PKT4(ring, REG_A6XX_SP_DS_CTRL_REG0, 1);
+		OUT_RING(ring, A6XX_SP_DS_CTRL_REG0_THREADSIZE(TWO_QUADS) |
+			A6XX_SP_DS_CTRL_REG0_FULLREGFOOTPRINT(ds->info.max_reg + 1) |
+			A6XX_SP_DS_CTRL_REG0_BRANCHSTACK(ds->branchstack) |
+			COND(ds->need_pixlod, A6XX_SP_DS_CTRL_REG0_PIXLODENABLE));
+
+		fd6_emit_shader(ring, ds);
+		ir3_emit_immediates(screen, ds, ring);
+		ir3_emit_link_map(screen, hs, ds, ring);
+
+		shader_info *hs_info = &hs->shader->nir->info;
+		OUT_PKT4(ring, REG_A6XX_PC_TESS_NUM_VERTEX, 1);
+		OUT_RING(ring, hs_info->tess.tcs_vertices_out);
+
+		/* Total attribute slots in HS incoming patch. */
+		OUT_PKT4(ring, REG_A6XX_PC_UNKNOWN_9801, 1);
+		OUT_RING(ring, hs_info->tess.tcs_vertices_out * vs->shader->output_size / 4);
+
+		OUT_PKT4(ring, REG_A6XX_SP_HS_UNKNOWN_A831, 1);
+		OUT_RING(ring, vs->shader->output_size);
+
+		shader_info *ds_info = &ds->shader->nir->info;
+		OUT_PKT4(ring, REG_A6XX_PC_TESS_CNTL, 1);
+		uint32_t output;
+		if (ds_info->tess.point_mode)
+			output = TESS_POINTS;
+		else if (ds_info->tess.primitive_mode == GL_ISOLINES)
+			output = TESS_LINES;
+		else if (ds_info->tess.ccw)
+			output = TESS_CCW_TRIS;
+		else
+			output = TESS_CW_TRIS;
+
+		OUT_RING(ring, A6XX_PC_TESS_CNTL_SPACING(fd6_gl2spacing(ds_info->tess.spacing)) |
+				A6XX_PC_TESS_CNTL_OUTPUT(output));
+
+		/* xxx: Misc tess unknowns: */
+		OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9103, 1);
+		OUT_RING(ring, 0x00ffff00);
+
+		OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9106, 1);
+		OUT_RING(ring, 0x0000ffff);
+
+		OUT_PKT4(ring, REG_A6XX_GRAS_UNKNOWN_809D, 1);
+		OUT_RING(ring, 0x0);
+
+		OUT_PKT4(ring, REG_A6XX_GRAS_UNKNOWN_8002, 1);
+		OUT_RING(ring, 0x0);
+
+		OUT_PKT4(ring, REG_A6XX_VPC_PACK, 1);
+		OUT_RING(ring, A6XX_VPC_PACK_POSITIONLOC(pos_loc) |
+				 A6XX_VPC_PACK_PSIZELOC(255) |
+				 A6XX_VPC_PACK_STRIDE_IN_VPC(l.max_loc));
+
+		OUT_PKT4(ring, REG_A6XX_VPC_PACK_3, 1);
+		OUT_RING(ring, A6XX_VPC_PACK_3_POSITIONLOC(pos_loc) |
+				 A6XX_VPC_PACK_3_PSIZELOC(psize_loc) |
+				 A6XX_VPC_PACK_3_STRIDE_IN_VPC(l.max_loc));
+
+		OUT_PKT4(ring, REG_A6XX_SP_DS_PRIMITIVE_CNTL, 1);
+		OUT_RING(ring, A6XX_SP_DS_PRIMITIVE_CNTL_DSOUT(l.cnt));
+
+		OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_4, 1);
+		OUT_RING(ring, A6XX_PC_PRIMITIVE_CNTL_4_STRIDE_IN_VPC(l.max_loc) |
+				CONDREG(psize_regid, 0x100));
+
+	} else {
+		OUT_PKT4(ring, REG_A6XX_SP_HS_UNKNOWN_A831, 1);
+		OUT_RING(ring, 0);
+	}
 
 	OUT_PKT4(ring, REG_A6XX_SP_PRIMITIVE_CNTL, 1);
 	OUT_RING(ring, A6XX_SP_PRIMITIVE_CNTL_VSOUT(l.cnt));
@@ -439,16 +602,10 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 
 	OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_1, 1);
 	OUT_RING(ring, A6XX_PC_PRIMITIVE_CNTL_1_STRIDE_IN_VPC(l.max_loc) |
-			 CONDREG(psize_regid, 0x100));
+			CONDREG(psize_regid, A6XX_PC_PRIMITIVE_CNTL_1_PSIZE));
 
-	if (binning_pass) {
-		OUT_PKT4(ring, REG_A6XX_SP_FS_OBJ_START_LO, 2);
-		OUT_RING(ring, 0x00000000);    /* SP_FS_OBJ_START_LO */
-		OUT_RING(ring, 0x00000000);    /* SP_FS_OBJ_START_HI */
-	} else {
-		OUT_PKT4(ring, REG_A6XX_SP_FS_OBJ_START_LO, 2);
-		OUT_RELOC(ring, fs->bo, 0, 0, 0);  /* SP_FS_OBJ_START_LO/HI */
-	}
+	OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_3, 1);
+	OUT_RING(ring, 0);
 
 	OUT_PKT4(ring, REG_A6XX_HLSQ_CONTROL_1_REG, 5);
 	OUT_RING(ring, 0x7);                /* XXX */
@@ -537,46 +694,120 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	}
 
 	OUT_PKT4(ring, REG_A6XX_VPC_PACK, 1);
-	OUT_RING(ring, A6XX_VPC_PACK_NUMNONPOSVAR(fs->total_in) |
+	OUT_RING(ring, A6XX_VPC_PACK_POSITIONLOC(pos_loc) |
 			 A6XX_VPC_PACK_PSIZELOC(psize_loc) |
 			 A6XX_VPC_PACK_STRIDE_IN_VPC(l.max_loc));
 
-	if (!binning_pass) {
-		/* figure out VARYING_INTERP / VARYING_PS_REPL register values: */
-		for (j = -1; (j = ir3_next_varying(fs, j)) < (int)fs->inputs_count; ) {
-			/* NOTE: varyings are packed, so if compmask is 0xb
-			 * then first, third, and fourth component occupy
-			 * three consecutive varying slots:
-			 */
-			unsigned compmask = fs->inputs[j].compmask;
+	if (gs) {
+		OUT_PKT4(ring, REG_A6XX_SP_GS_CTRL_REG0, 1);
+		OUT_RING(ring, A6XX_SP_GS_CTRL_REG0_THREADSIZE(TWO_QUADS) |
+			A6XX_SP_GS_CTRL_REG0_FULLREGFOOTPRINT(gs->info.max_reg + 1) |
+			A6XX_SP_GS_CTRL_REG0_BRANCHSTACK(gs->branchstack) |
+			COND(gs->need_pixlod, A6XX_SP_GS_CTRL_REG0_PIXLODENABLE));
 
-			uint32_t inloc = fs->inputs[j].inloc;
+		fd6_emit_shader(ring, gs);
+		ir3_emit_immediates(screen, gs, ring);
+		if (ds)
+			ir3_emit_link_map(screen, ds, gs, ring);
+		else
+			ir3_emit_link_map(screen, vs, gs, ring);
 
-			if (fs->inputs[j].interpolate == INTERP_MODE_FLAT) {
-				uint32_t loc = inloc;
+		OUT_PKT4(ring, REG_A6XX_VPC_PACK_GS, 1);
+		OUT_RING(ring, A6XX_VPC_PACK_GS_POSITIONLOC(pos_loc) |
+				 A6XX_VPC_PACK_GS_PSIZELOC(psize_loc) |
+				 A6XX_VPC_PACK_GS_STRIDE_IN_VPC(l.max_loc));
 
-				for (i = 0; i < 4; i++) {
-					if (compmask & (1 << i)) {
-						state->vinterp[loc / 16] |= 1 << ((loc % 16) * 2);
-						loc++;
-					}
-				}
-			}
+		OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9105, 1);
+		OUT_RING(ring, A6XX_VPC_UNKNOWN_9105_LAYERLOC(layer_loc) | 0xff00);
+
+		OUT_PKT4(ring, REG_A6XX_GRAS_UNKNOWN_809C, 1);
+		OUT_RING(ring, CONDREG(layer_regid, A6XX_GRAS_UNKNOWN_809C_GS_WRITES_LAYER));
+
+		uint32_t flags_regid = ir3_find_output_regid(gs, VARYING_SLOT_GS_VERTEX_FLAGS_IR3);
+
+		OUT_PKT4(ring, REG_A6XX_SP_PRIMITIVE_CNTL_GS, 1);
+		OUT_RING(ring, A6XX_SP_PRIMITIVE_CNTL_GS_GSOUT(l.cnt) |
+				A6XX_SP_PRIMITIVE_CNTL_GS_FLAGS_REGID(flags_regid));
+
+		OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_2, 1);
+		OUT_RING(ring, A6XX_PC_PRIMITIVE_CNTL_2_STRIDE_IN_VPC(l.max_loc) |
+				CONDREG(psize_regid, A6XX_PC_PRIMITIVE_CNTL_2_PSIZE) |
+				CONDREG(layer_regid, A6XX_PC_PRIMITIVE_CNTL_2_LAYER) |
+				CONDREG(primitive_regid, A6XX_PC_PRIMITIVE_CNTL_2_PRIMITIVE_ID));
+
+		uint32_t output;
+		switch (gs->shader->nir->info.gs.output_primitive) {
+		case GL_POINTS:
+			output = TESS_POINTS;
+			break;
+		case GL_LINE_STRIP:
+			output = TESS_LINES;
+			break;
+		case GL_TRIANGLE_STRIP:
+			output = TESS_CW_TRIS;
+			break;
+		default:
+			unreachable("");
 		}
+		OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_5, 1);
+		OUT_RING(ring,
+				A6XX_PC_PRIMITIVE_CNTL_5_GS_VERTICES_OUT(gs->shader->nir->info.gs.vertices_out - 1) |
+				A6XX_PC_PRIMITIVE_CNTL_5_GS_OUTPUT(output) |
+				A6XX_PC_PRIMITIVE_CNTL_5_GS_INVOCATIONS(gs->shader->nir->info.gs.invocations - 1));
+
+		OUT_PKT4(ring, REG_A6XX_GRAS_UNKNOWN_8003, 1);
+		OUT_RING(ring, 0);
+
+		OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9100, 1);
+		OUT_RING(ring, 0xff);
+
+		OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9102, 1);
+		OUT_RING(ring, 0xffff00);
+
+		const struct ir3_shader_variant *prev = state->ds ? state->ds : state->vs;
+
+		/* Size of per-primitive alloction in ldlw memory in vec4s. */
+		uint32_t vec4_size =
+			gs->shader->nir->info.gs.vertices_in *
+			DIV_ROUND_UP(prev->shader->output_size, 4);
+		OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_6, 1);
+		OUT_RING(ring, A6XX_PC_PRIMITIVE_CNTL_6_STRIDE_IN_VPC(vec4_size));
+
+		OUT_PKT4(ring, REG_A6XX_PC_UNKNOWN_9B07, 1);
+		OUT_RING(ring, 0);
+
+		OUT_PKT4(ring, REG_A6XX_SP_GS_UNKNOWN_A871, 1);
+		OUT_RING(ring, prev->shader->output_size);
+	} else {
+		OUT_PKT4(ring, REG_A6XX_PC_PRIMITIVE_CNTL_6, 1);
+		OUT_RING(ring, 0);
+		OUT_PKT4(ring, REG_A6XX_SP_GS_UNKNOWN_A871, 1);
+		OUT_RING(ring, 0);
 	}
 
-	if (!binning_pass)
-		if (fs->instrlen)
-			fd6_emit_shader(ring, fs);
+	OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9101, 1);
+	OUT_RING(ring, 0xffff00);
+
+	OUT_PKT4(ring, REG_A6XX_VPC_UNKNOWN_9107, 1);
+	OUT_RING(ring, 0);
+
+	if (fs->instrlen)
+		fd6_emit_shader(ring, fs);
 
 	OUT_PKT4(ring, REG_A6XX_VFD_CONTROL_1, 6);
 	OUT_RING(ring, A6XX_VFD_CONTROL_1_REGID4VTX(vertex_regid) |
 			A6XX_VFD_CONTROL_1_REGID4INST(instance_regid) |
-			0xfcfc0000);
-	OUT_RING(ring, 0x0000fcfc);   /* VFD_CONTROL_2 */
-	OUT_RING(ring, 0xfcfcfcfc);   /* VFD_CONTROL_3 */
+			A6XX_VFD_CONTROL_1_REGID4PRIMID(primitive_regid) |
+			0xfc000000);
+	OUT_RING(ring, A6XX_VFD_CONTROL_2_REGID_HSPATCHID(hs_patch_regid) |
+			A6XX_VFD_CONTROL_2_REGID_INVOCATIONID(hs_invocation_regid));
+	OUT_RING(ring, A6XX_VFD_CONTROL_3_REGID_DSPATCHID(ds_patch_regid) |
+			A6XX_VFD_CONTROL_3_REGID_TESSX(tess_coord_x_regid) |
+			A6XX_VFD_CONTROL_3_REGID_TESSY(tess_coord_y_regid) |
+			0xfc);
 	OUT_RING(ring, 0x000000fc);   /* VFD_CONTROL_4 */
-	OUT_RING(ring, 0x0000fcfc);   /* VFD_CONTROL_5 */
+	OUT_RING(ring, A6XX_VFD_CONTROL_5_REGID_GSHEADER(gs_header_regid) |
+			0xfc00);   /* VFD_CONTROL_5 */
 	OUT_RING(ring, 0x00000000);   /* VFD_CONTROL_6 */
 
 	bool fragz = fs->no_earlyz | fs->writes_pos;
@@ -587,29 +818,66 @@ setup_stateobj(struct fd_ringbuffer *ring, struct fd_screen *screen,
 	OUT_PKT4(ring, REG_A6XX_GRAS_SU_DEPTH_PLANE_CNTL, 1);
 	OUT_RING(ring, COND(fragz, A6XX_GRAS_SU_DEPTH_PLANE_CNTL_FRAG_WRITES_Z));
 
-	ir3_emit_immediates(screen, vs, ring);
 	if (!binning_pass)
 		ir3_emit_immediates(screen, fs, ring);
 }
 
-/* emits the program state which is not part of the stateobj because of
- * dependency on other gl state (rasterflat or sprite-coord-replacement)
+static struct fd_ringbuffer *
+create_interp_stateobj(struct fd_context *ctx, struct fd6_program_state *state)
+{
+	const struct ir3_shader_variant *fs = state->fs;
+	struct fd_ringbuffer *ring = fd_ringbuffer_new_object(ctx->pipe, 18 * 4);
+	uint32_t vinterp[8] = {0};
+
+	/* figure out VARYING_INTERP / VARYING_PS_REPL register values: */
+	for (int j = -1; (j = ir3_next_varying(fs, j)) < (int)fs->inputs_count; ) {
+		/* NOTE: varyings are packed, so if compmask is 0xb
+		 * then first, third, and fourth component occupy
+		 * three consecutive varying slots:
+		 */
+		unsigned compmask = fs->inputs[j].compmask;
+
+		uint32_t inloc = fs->inputs[j].inloc;
+
+		if (fs->inputs[j].interpolate == INTERP_MODE_FLAT) {
+			uint32_t loc = inloc;
+
+			for (int i = 0; i < 4; i++) {
+				if (compmask & (1 << i)) {
+					vinterp[loc / 16] |= 1 << ((loc % 16) * 2);
+					loc++;
+				}
+			}
+		}
+	}
+
+	OUT_PKT4(ring, REG_A6XX_VPC_VARYING_INTERP_MODE(0), 8);
+	for (int i = 0; i < 8; i++)
+		OUT_RING(ring, vinterp[i]);    /* VPC_VARYING_INTERP[i].MODE */
+
+	OUT_PKT4(ring, REG_A6XX_VPC_VARYING_PS_REPL_MODE(0), 8);
+	for (int i = 0; i < 8; i++)
+		OUT_RING(ring, 0x00000000);    /* VPC_VARYING_PS_REPL[i] */
+
+	return ring;
+}
+
+/* build the program streaming state which is not part of the pre-
+ * baked stateobj because of dependency on other gl state (rasterflat
+ * or sprite-coord-replacement)
  */
-void
-fd6_program_emit(struct fd_ringbuffer *ring, struct fd6_emit *emit)
+struct fd_ringbuffer *
+fd6_program_interp_state(struct fd6_emit *emit)
 {
 	const struct fd6_program_state *state = fd6_emit_get_prog(emit);
 
 	if (!unlikely(emit->rasterflat || emit->sprite_coord_enable)) {
 		/* fastpath: */
-		OUT_PKT4(ring, REG_A6XX_VPC_VARYING_INTERP_MODE(0), 8);
-		for (int i = 0; i < 8; i++)
-			OUT_RING(ring, state->vinterp[i]);   /* VPC_VARYING_INTERP[i].MODE */
-
-		OUT_PKT4(ring, REG_A6XX_VPC_VARYING_PS_REPL_MODE(0), 8);
-		for (int i = 0; i < 8; i++)
-			OUT_RING(ring, 0x00000000);          /* VPC_VARYING_PS_REPL[i] */
+		return fd_ringbuffer_ref(state->interp_stateobj);
 	} else {
+		struct fd_ringbuffer *ring = fd_submit_new_ringbuffer(
+				emit->ctx->batch->submit, 18 * 4, FD_RINGBUFFER_STREAMING);
+
 		/* slow-path: */
 		struct ir3_shader_variant *fs = state->fs;
 		uint32_t vinterp[8], vpsrepl[8];
@@ -684,36 +952,51 @@ fd6_program_emit(struct fd_ringbuffer *ring, struct fd6_emit *emit)
 		OUT_PKT4(ring, REG_A6XX_VPC_VARYING_PS_REPL_MODE(0), 8);
 		for (int i = 0; i < 8; i++)
 			OUT_RING(ring, vpsrepl[i]);     /* VPC_VARYING_PS_REPL[i] */
+
+		return ring;
 	}
 }
 
 static struct ir3_program_state *
 fd6_program_create(void *data, struct ir3_shader_variant *bs,
 		struct ir3_shader_variant *vs,
+		struct ir3_shader_variant *hs,
+		struct ir3_shader_variant *ds,
+		struct ir3_shader_variant *gs,
 		struct ir3_shader_variant *fs,
 		const struct ir3_shader_key *key)
 {
 	struct fd_context *ctx = data;
 	struct fd6_program_state *state = CALLOC_STRUCT(fd6_program_state);
 
-	state->bs = bs;
+	/* if we have streamout, use full VS in binning pass, as the
+	 * binning pass VS will have outputs on other than position/psize
+	 * stripped out:
+	 */
+	state->bs = vs->shader->stream_output.num_outputs ? vs : bs;
 	state->vs = vs;
+	state->hs = hs;
+	state->ds = ds;
+	state->gs = gs;
 	state->fs = fs;
 	state->config_stateobj = fd_ringbuffer_new_object(ctx->pipe, 0x1000);
 	state->binning_stateobj = fd_ringbuffer_new_object(ctx->pipe, 0x1000);
 	state->stateobj = fd_ringbuffer_new_object(ctx->pipe, 0x1000);
 
 #ifdef DEBUG
-	for (unsigned i = 0; i < bs->inputs_count; i++) {
-		if (vs->inputs[i].sysval)
-			continue;
-		debug_assert(bs->inputs[i].regid == vs->inputs[i].regid);
+	if (!ds) {
+		for (unsigned i = 0; i < bs->inputs_count; i++) {
+			if (vs->inputs[i].sysval)
+				continue;
+			debug_assert(bs->inputs[i].regid == vs->inputs[i].regid);
+		}
 	}
 #endif
 
 	setup_config_stateobj(state->config_stateobj, state);
 	setup_stateobj(state->binning_stateobj, ctx->screen, state, key, true);
 	setup_stateobj(state->stateobj, ctx->screen, state, key, false);
+	state->interp_stateobj = create_interp_stateobj(ctx, state);
 
 	return &state->base;
 }
@@ -725,6 +1008,7 @@ fd6_program_destroy(void *data, struct ir3_program_state *state)
 	fd_ringbuffer_del(so->stateobj);
 	fd_ringbuffer_del(so->binning_stateobj);
 	fd_ringbuffer_del(so->config_stateobj);
+	fd_ringbuffer_del(so->interp_stateobj);
 	free(so);
 }
 
@@ -733,6 +1017,37 @@ static const struct ir3_cache_funcs cache_funcs = {
 	.destroy_state = fd6_program_destroy,
 };
 
+static void *
+fd6_shader_state_create(struct pipe_context *pctx, const struct pipe_shader_state *cso)
+{
+	struct fd_context *ctx = fd_context(pctx);
+	struct ir3_compiler *compiler = ctx->screen->compiler;
+	struct ir3_shader *shader =
+		ir3_shader_create(compiler, cso, &ctx->debug, pctx->screen);
+	unsigned packets, size;
+
+	/* pre-calculate size required for userconst stateobj: */
+	ir3_user_consts_size(&shader->ubo_state, &packets, &size);
+
+	/* also account for UBO addresses: */
+	packets += 1;
+	size += 2 * align(shader->const_state.num_ubos, 2);
+
+	unsigned sizedwords = (4 * packets) + size;
+	shader->ubo_state.cmdstream_size = sizedwords * 4;
+
+	return shader;
+}
+
+static void
+fd6_shader_state_delete(struct pipe_context *pctx, void *hwcso)
+{
+	struct ir3_shader *so = hwcso;
+	struct fd_context *ctx = fd_context(pctx);
+	ir3_cache_invalidate(fd6_context(ctx)->shader_cache, hwcso);
+	ir3_shader_destroy(so);
+}
+
 void
 fd6_prog_init(struct pipe_context *pctx)
 {
@@ -740,11 +1055,23 @@ fd6_prog_init(struct pipe_context *pctx)
 
 	fd6_context(ctx)->shader_cache = ir3_cache_create(&cache_funcs, ctx);
 
-	pctx->create_fs_state = fd6_fp_state_create;
-	pctx->delete_fs_state = fd6_fp_state_delete;
+	pctx->create_vs_state = fd6_shader_state_create;
+	pctx->delete_vs_state = fd6_shader_state_delete;
 
-	pctx->create_vs_state = fd6_vp_state_create;
-	pctx->delete_vs_state = fd6_vp_state_delete;
+	pctx->create_tcs_state = fd6_shader_state_create;
+	pctx->delete_tcs_state = fd6_shader_state_delete;
+
+	pctx->create_tes_state = fd6_shader_state_create;
+	pctx->delete_tes_state = fd6_shader_state_delete;
+
+	pctx->create_gs_state = fd6_shader_state_create;
+	pctx->delete_gs_state = fd6_shader_state_delete;
+
+	pctx->create_gs_state = fd6_shader_state_create;
+	pctx->delete_gs_state = fd6_shader_state_delete;
+
+	pctx->create_fs_state = fd6_shader_state_create;
+	pctx->delete_fs_state = fd6_shader_state_delete;
 
 	fd_prog_init(pctx);
 }
