@@ -27,7 +27,7 @@
 
 #include "drm-uapi/drm_fourcc.h"
 #include "pipe/p_screen.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 
 #include "fd6_screen.h"
 #include "fd6_blitter.h"
@@ -82,9 +82,9 @@ fd6_screen_is_format_supported(struct pipe_screen *pscreen,
 	}
 
 	if ((usage & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_SHADER_IMAGE)) &&
+			(fd6_pipe2tex(format) != (enum a6xx_tex_fmt)~0) &&
 			(target == PIPE_BUFFER ||
-			 util_format_get_blocksize(format) != 12) &&
-			(fd6_pipe2tex(format) != (enum a6xx_tex_fmt)~0)) {
+			 util_format_get_blocksize(format) != 12)) {
 		retval |= usage & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_SHADER_IMAGE);
 	}
 
@@ -127,9 +127,6 @@ fd6_screen_is_format_supported(struct pipe_screen *pscreen,
 	return retval == usage;
 }
 
-extern const struct fd_perfcntr_group a6xx_perfcntr_groups[];
-extern const unsigned a6xx_num_perfcntr_groups;
-
 void
 fd6_screen_init(struct pipe_screen *pscreen)
 {
@@ -151,10 +148,5 @@ fd6_screen_init(struct pipe_screen *pscreen)
 	screen->supported_modifiers = supported_modifiers;
 	screen->num_supported_modifiers = ARRAY_SIZE(supported_modifiers);
 
-	if (fd_mesa_debug & FD_DBG_PERFC) {
-		screen->perfcntr_groups = a6xx_perfcntr_groups;
-		screen->num_perfcntr_groups = a6xx_num_perfcntr_groups;
-	}
-
 	fd6_emit_init_screen(pscreen);
 }
