@@ -517,18 +517,23 @@ iris_get_timestamp(struct pipe_screen *pscreen)
    return result;
 }
 
-static void
-iris_destroy_screen(struct pipe_screen *pscreen)
+void
+iris_screen_destroy(struct iris_screen *screen)
 {
-   struct iris_screen *screen = (struct iris_screen *) pscreen;
    iris_bo_unreference(screen->workaround_bo);
-   u_transfer_helper_destroy(pscreen->transfer_helper);
-   iris_bufmgr_destroy(screen->bufmgr);
+   u_transfer_helper_destroy(screen->base.transfer_helper);
+   iris_bufmgr_unref(screen->bufmgr);
    disk_cache_destroy(screen->disk_cache);
-   close(screen->fd);
+   close(screen->winsys_fd);
    ralloc_free(screen);
 }
 
+static void
+iris_screen_unref(struct pipe_screen *pscreen)
+{
+   iris_pscreen_unref(pscreen);
+}
+
 static void
 iris_query_memory_info(struct pipe_screen *pscreen,
                        struct pipe_memory_info *info)
@@ -555,22 +560,22 @@ iris_get_disk_shader_cache(struct pipe_screen *pscreen)
 }
 
 static int
-iris_getparam(struct iris_screen *screen, int param, int *value)
+iris_getparam(int fd, int param, int *value)
 {
    struct drm_i915_getparam gp = { .param = param, .value = value };
 
-   if (ioctl(screen->fd, DRM_IOCTL_I915_GETPARAM, &gp) == -1)
+   if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp) == -1)
       return -errno;
 
    return 0;
 }
 
 static int
-iris_getparam_integer(struct iris_screen *screen, int param)
+iris_getparam_integer(int fd, int param)
 {
    int value = -1;
 
-   if (iris_getparam(screen, param, &value) == 0)
+   if (iris_getparam(fd, param, &value) == 0)
       return value;
 
    return -1;
@@ -627,25 +632,34 @@ iris_shader_perf_log(void *data, const char *fmt, ...)
 struct pipe_screen *
 iris_screen_create(int fd, const struct pipe_screen_config *config)
 {
+   /* Here are the i915 features we need for Iris (in chronoligical order) :
+    *    - I915_PARAM_HAS_EXEC_NO_RELOC     (3.10)
+    *    - I915_PARAM_HAS_EXEC_HANDLE_LUT   (3.10)
+    *    - I915_PARAM_HAS_EXEC_BATCH_FIRST  (4.13)
+    *    - I915_PARAM_HAS_EXEC_FENCE_ARRAY  (4.14)
+    *    - I915_PARAM_HAS_CONTEXT_ISOLATION (4.16)
+    *
+    * Checking the last feature availability will include all previous ones.
+    */
+   if (!iris_getparam_integer(fd, I915_PARAM_HAS_CONTEXT_ISOLATION)) {
+      debug_error("Kernel is too old for Iris. Consider upgrading to kernel v4.16.\n");
+      return NULL;
+   }
+
    struct iris_screen *screen = rzalloc(NULL, struct iris_screen);
    if (!screen)
       return NULL;
 
-   screen->fd = fd;
-
    if (!gen_get_device_info_from_fd(fd, &screen->devinfo))
       return NULL;
    screen->pci_id = screen->devinfo.chipset_id;
    screen->no_hw = screen->devinfo.no_hw;
 
+   p_atomic_set(&screen->refcount, 1);
+
    if (screen->devinfo.gen < 8 || screen->devinfo.is_cherryview)
       return NULL;
 
-   screen->aperture_bytes = get_aperture_size(fd);
-
-   if (getenv("INTEL_NO_HW") != NULL)
-      screen->no_hw = true;
-
    bool bo_reuse = false;
    int bo_reuse_mode = driQueryOptioni(config->options, "bo_reuse");
    switch (bo_reuse_mode) {
@@ -656,10 +670,18 @@ iris_screen_create(int fd, const struct pipe_screen_config *config)
       break;
    }
 
-   screen->bufmgr = iris_bufmgr_init(&screen->devinfo, fd, bo_reuse);
+   screen->bufmgr = iris_bufmgr_get_for_fd(&screen->devinfo, fd, bo_reuse);
    if (!screen->bufmgr)
       return NULL;
 
+   screen->fd = iris_bufmgr_get_fd(screen->bufmgr);
+   screen->winsys_fd = fd;
+
+   screen->aperture_bytes = get_aperture_size(fd);
+
+   if (getenv("INTEL_NO_HW") != NULL)
+      screen->no_hw = true;
+
    screen->workaround_bo =
       iris_bo_alloc(screen->bufmgr, "workaround", 4096, IRIS_MEMZONE_OTHER);
    if (!screen->workaround_bo)
@@ -694,7 +716,7 @@ iris_screen_create(int fd, const struct pipe_screen_config *config)
                       sizeof(struct iris_transfer), 64);
 
    screen->subslice_total =
-      iris_getparam_integer(screen, I915_PARAM_SUBSLICE_TOTAL);
+      iris_getparam_integer(screen->fd, I915_PARAM_SUBSLICE_TOTAL);
    assert(screen->subslice_total >= 1);
 
    struct pipe_screen *pscreen = &screen->base;
@@ -702,7 +724,7 @@ iris_screen_create(int fd, const struct pipe_screen_config *config)
    iris_init_screen_fence_functions(pscreen);
    iris_init_screen_resource_functions(pscreen);
 
-   pscreen->destroy = iris_destroy_screen;
+   pscreen->destroy = iris_screen_unref;
    pscreen->get_name = iris_get_name;
    pscreen->get_vendor = iris_get_vendor;
    pscreen->get_device_vendor = iris_get_device_vendor;
