@@ -23,9 +23,8 @@
 
 /* r300_emit: Functions for emitting state. */
 
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_math.h"
-#include "util/u_mm.h"
 
 #include "r300_context.h"
 #include "r300_cb.h"
