@@ -25,7 +25,7 @@
 #include "si_pipe.h"
 #include "sid.h"
 
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_pack_color.h"
 #include "util/u_surface.h"
 
@@ -245,7 +245,7 @@ bool vi_dcc_clear_level(struct si_context *sctx,
 		dcc_offset = 0;
 	} else {
 		dcc_buffer = &tex->buffer.b.b;
-		dcc_offset = tex->dcc_offset;
+		dcc_offset = tex->surface.dcc_offset;
 	}
 
 	if (sctx->chip_class >= GFX9) {
@@ -400,7 +400,7 @@ static void si_do_fast_color_clear(struct si_context *sctx,
 	int i;
 
 	/* This function is broken in BE, so just disable this path for now */
-#ifdef PIPE_ARCH_BIG_ENDIAN
+#if UTIL_ARCH_BIG_ENDIAN
 	return;
 #endif
 
@@ -510,12 +510,13 @@ static void si_do_fast_color_clear(struct si_context *sctx,
 				continue;
 
 			tex->separate_dcc_dirty = true;
+			tex->displayable_dcc_dirty = true;
 
 			/* DCC fast clear with MSAA should clear CMASK to 0xC. */
 			if (tex->buffer.b.b.nr_samples >= 2 && tex->cmask_buffer) {
 				uint32_t clear_value = 0xCCCCCCCC;
 				si_clear_buffer(sctx, &tex->cmask_buffer->b.b,
-						tex->cmask_offset, tex->surface.cmask_size,
+						tex->surface.cmask_offset, tex->surface.cmask_size,
 						&clear_value, 4, SI_COHERENCY_CB_META, false);
 				fmask_decompress_needed = true;
 			}
@@ -540,7 +541,7 @@ static void si_do_fast_color_clear(struct si_context *sctx,
 			/* Do the fast clear. */
 			uint32_t clear_value = 0;
 			si_clear_buffer(sctx, &tex->cmask_buffer->b.b,
-					tex->cmask_offset, tex->surface.cmask_size,
+					tex->surface.cmask_offset, tex->surface.cmask_size,
 					&clear_value, 4, SI_COHERENCY_CB_META, false);
 			eliminate_needed = true;
 		}
@@ -559,7 +560,7 @@ static void si_do_fast_color_clear(struct si_context *sctx,
 		/* Chips with DCC constant encoding don't need to set the clear
 		 * color registers for DCC clear values 0 and 1.
 		 */
-		if (sctx->screen->has_dcc_constant_encode && !eliminate_needed)
+		if (sctx->screen->info.has_dcc_constant_encode && !eliminate_needed)
 			continue;
 
 		if (si_set_clear_color(tex, fb->cbufs[i]->format, color)) {
@@ -692,7 +693,7 @@ static void si_clear_render_target(struct pipe_context *ctx,
 	struct si_context *sctx = (struct si_context *)ctx;
 	struct si_texture *sdst = (struct si_texture*)dst->texture;
 
-	if (dst->texture->nr_samples <= 1 && !sdst->dcc_offset) {
+	if (dst->texture->nr_samples <= 1 && !sdst->surface.dcc_offset) {
 		si_compute_clear_render_target(ctx, dst, color, dstx, dsty, width,
 					       height, render_condition_enabled);
 		return;
