@@ -106,7 +106,6 @@ enum {
 	SI_QUERY_NUM_COMPILATIONS,
 	SI_QUERY_NUM_SHADERS_CREATED,
 	SI_QUERY_BACK_BUFFER_PS_DRAW_RATIO,
-	SI_QUERY_NUM_SHADER_CACHE_HITS,
 	SI_QUERY_GPIN_ASIC_ID,
 	SI_QUERY_GPIN_NUM_SIMD,
 	SI_QUERY_GPIN_NUM_RB,
@@ -117,6 +116,12 @@ enum {
 	SI_QUERY_PD_NUM_PRIMS_ACCEPTED,
 	SI_QUERY_PD_NUM_PRIMS_REJECTED,
 	SI_QUERY_PD_NUM_PRIMS_INELIGIBLE,
+	SI_QUERY_LIVE_SHADER_CACHE_HITS,
+	SI_QUERY_LIVE_SHADER_CACHE_MISSES,
+	SI_QUERY_MEMORY_SHADER_CACHE_HITS,
+	SI_QUERY_MEMORY_SHADER_CACHE_MISSES,
+	SI_QUERY_DISK_SHADER_CACHE_HITS,
+	SI_QUERY_DISK_SHADER_CACHE_MISSES,
 
 	SI_QUERY_FIRST_PERFCOUNTER = PIPE_QUERY_DRIVER_SPECIFIC + 100,
 };
