@@ -25,6 +25,7 @@
 
 #include "svga_context.h"
 #include "svga_link.h"
+#include "svga_debug.h"
 
 #include "tgsi/tgsi_strings.h"
 
@@ -61,7 +62,7 @@ svga_link_shaders(const struct tgsi_shader_info *outshader_info,
    free_slot = outshader_info->num_outputs + 1;
 
    for (i = 0; i < inshader_info->num_inputs; i++) {
-      unsigned sem_name = inshader_info->input_semantic_name[i];
+      enum tgsi_semantic sem_name = inshader_info->input_semantic_name[i];
       unsigned sem_index = inshader_info->input_semantic_index[i];
       unsigned j;
       /**
@@ -95,15 +96,19 @@ svga_link_shaders(const struct tgsi_shader_info *outshader_info,
          linkage->input_map[i] = j;
       }
    }
+   linkage->input_map_max = free_slot - 1;
 
    /* Debug */
-   if (0) {
+   if (SVGA_DEBUG & DEBUG_TGSI) {
       unsigned reg = 0;
+      debug_printf("### linkage info: num_inputs=%d input_map_max=%d\n",
+                   linkage->num_inputs, linkage->input_map_max);
+
       for (i = 0; i < linkage->num_inputs; i++) {
 
          assert(linkage->input_map[i] != INVALID_INDEX);
 
-         debug_printf("input shader input[%d] slot %u  %s %u %s\n",
+         debug_printf("   input[%d] slot %u  %s %u %s\n",
                       i,
                       linkage->input_map[i],
                       tgsi_semantic_names[inshader_info->input_semantic_name[i]],
