@@ -322,7 +322,7 @@ svga_buffer_add_host_surface(struct svga_buffer *sbuf,
    bufsurf->key = *key;
 
    /* add the surface to the surface list */
-   LIST_ADD(&bufsurf->list, &sbuf->surfaces);
+   list_add(&bufsurf->list, &sbuf->surfaces);
 
    /* Set the new bind flags for this buffer resource */
    sbuf->bind_flags = bind_flags;
@@ -410,7 +410,7 @@ svga_buffer_validate_host_surface(struct svga_context *svga,
          svga_screen_surface_destroy(svga_screen(sbuf->b.b.screen),
                                      &bufsurf->key, &bufsurf->handle);
 
-         LIST_DEL(&bufsurf->list);
+         list_del(&bufsurf->list);
          FREE(bufsurf);
       }
    } else {
@@ -728,7 +728,7 @@ svga_buffer_upload_flush(struct svga_context *svga, struct svga_buffer *sbuf)
    sbuf->map.num_ranges = 0;
 
    assert(sbuf->head.prev && sbuf->head.next);
-   LIST_DEL(&sbuf->head);  /* remove from svga->dirty_buffers list */
+   list_del(&sbuf->head);  /* remove from svga->dirty_buffers list */
 #ifdef DEBUG
    sbuf->head.next = sbuf->head.prev = NULL;
 #endif
@@ -1065,7 +1065,7 @@ svga_buffer_handle(struct svga_context *svga, struct pipe_resource *buf,
             if (ret == PIPE_OK) {
                sbuf->dma.pending = TRUE;
                assert(!sbuf->head.prev && !sbuf->head.next);
-               LIST_ADDTAIL(&sbuf->head, &svga->dirty_buffers);
+               list_addtail(&sbuf->head, &svga->dirty_buffers);
             }
          }
          else if (ret == PIPE_ERROR_OUT_OF_MEMORY) {
