@@ -39,7 +39,9 @@ enum pipe_video_format
    PIPE_VIDEO_FORMAT_MPEG4,    /**< DIVX, XVID */
    PIPE_VIDEO_FORMAT_VC1,      /**< WMV */
    PIPE_VIDEO_FORMAT_MPEG4_AVC,/**< H.264 */
-   PIPE_VIDEO_FORMAT_HEVC      /**< H.265 */
+   PIPE_VIDEO_FORMAT_HEVC,     /**< H.265 */
+   PIPE_VIDEO_FORMAT_JPEG,     /**< JPEG */
+   PIPE_VIDEO_FORMAT_VP9       /**< VP9 */
 };
 
 enum pipe_video_profile
@@ -54,6 +56,7 @@ enum pipe_video_profile
    PIPE_VIDEO_PROFILE_VC1_MAIN,
    PIPE_VIDEO_PROFILE_VC1_ADVANCED,
    PIPE_VIDEO_PROFILE_MPEG4_AVC_BASELINE,
+   PIPE_VIDEO_PROFILE_MPEG4_AVC_CONSTRAINED_BASELINE,
    PIPE_VIDEO_PROFILE_MPEG4_AVC_MAIN,
    PIPE_VIDEO_PROFILE_MPEG4_AVC_EXTENDED,
    PIPE_VIDEO_PROFILE_MPEG4_AVC_HIGH,
@@ -64,7 +67,11 @@ enum pipe_video_profile
    PIPE_VIDEO_PROFILE_HEVC_MAIN_10,
    PIPE_VIDEO_PROFILE_HEVC_MAIN_STILL,
    PIPE_VIDEO_PROFILE_HEVC_MAIN_12,
-   PIPE_VIDEO_PROFILE_HEVC_MAIN_444
+   PIPE_VIDEO_PROFILE_HEVC_MAIN_444,
+   PIPE_VIDEO_PROFILE_JPEG_BASELINE,
+   PIPE_VIDEO_PROFILE_VP9_PROFILE0,
+   PIPE_VIDEO_PROFILE_VP9_PROFILE2,
+   PIPE_VIDEO_PROFILE_MAX
 };
 
 /* Video caps, can be different for each codec/profile */
