@@ -34,15 +34,12 @@
 #include "dri_drawable.h"
 
 #include "pipe/p_screen.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_memory.h"
 #include "util/u_inlines.h"
 
 static uint32_t drifb_ID = 0;
 
-static void
-swap_fences_unref(struct dri_drawable *draw);
-
 static bool
 dri_st_framebuffer_validate(struct st_context_iface *stctx,
                             struct st_framebuffer_iface *stfbi,
@@ -179,9 +176,6 @@ dri_create_buffer(__DRIscreen * sPriv,
    drawable->screen = screen;
    drawable->sPriv = sPriv;
    drawable->dPriv = dPriv;
-   drawable->desired_fences = screen->default_throttle_frames;
-   if (drawable->desired_fences > DRI_SWAP_FENCES_MAX)
-      drawable->desired_fences = DRI_SWAP_FENCES_MAX;
 
    dPriv->driverPrivate = (void *)drawable;
    p_atomic_set(&drawable->base.stamp, 1);
@@ -209,7 +203,8 @@ dri_destroy_buffer(__DRIdrawable * dPriv)
    for (i = 0; i < ST_ATTACHMENT_COUNT; i++)
       pipe_resource_reference(&drawable->msaa_textures[i], NULL);
 
-   swap_fences_unref(drawable);
+   screen->base.screen->fence_reference(screen->base.screen,
+         &drawable->throttle_fence, NULL);
 
    /* Notify the st manager that this drawable is no longer valid */
    stapi->destroy_drawable(stapi, &drawable->base);
@@ -273,6 +268,9 @@ dri_set_tex_buffer2(__DRIcontext *pDRICtx, GLint target,
       if (format == __DRI_TEXTURE_FORMAT_RGB)  {
          /* only need to cover the formats recognized by dri_fill_st_visual */
          switch (internal_format) {
+         case PIPE_FORMAT_R16G16B16A16_FLOAT:
+            internal_format = PIPE_FORMAT_R16G16B16X16_FLOAT;
+            break;
          case PIPE_FORMAT_B10G10R10A2_UNORM:
             internal_format = PIPE_FORMAT_B10G10R10X2_UNORM;
             break;
@@ -346,75 +344,6 @@ dri_drawable_get_format(struct dri_drawable *drawable,
    }
 }
 
-
-/**
- * swap_fences_pop_front - pull a fence from the throttle queue
- *
- * If the throttle queue is filled to the desired number of fences,
- * pull fences off the queue until the number is less than the desired
- * number of fences, and return the last fence pulled.
- */
-static struct pipe_fence_handle *
-swap_fences_pop_front(struct dri_drawable *draw)
-{
-   struct pipe_screen *screen = draw->screen->base.screen;
-   struct pipe_fence_handle *fence = NULL;
-
-   if (draw->desired_fences == 0)
-      return NULL;
-
-   if (draw->cur_fences >= draw->desired_fences) {
-      screen->fence_reference(screen, &fence, draw->swap_fences[draw->tail]);
-      screen->fence_reference(screen, &draw->swap_fences[draw->tail++], NULL);
-      draw->tail &= DRI_SWAP_FENCES_MASK;
-      --draw->cur_fences;
-   }
-   return fence;
-}
-
-
-/**
- * swap_fences_push_back - push a fence onto the throttle queue
- *
- * push a fence onto the throttle queue and pull fences of the queue
- * so that the desired number of fences are on the queue.
- */
-static void
-swap_fences_push_back(struct dri_drawable *draw,
-		      struct pipe_fence_handle *fence)
-{
-   struct pipe_screen *screen = draw->screen->base.screen;
-
-   if (!fence || draw->desired_fences == 0)
-      return;
-
-   while(draw->cur_fences == draw->desired_fences)
-      swap_fences_pop_front(draw);
-
-   draw->cur_fences++;
-   screen->fence_reference(screen, &draw->swap_fences[draw->head++],
-			   fence);
-   draw->head &= DRI_SWAP_FENCES_MASK;
-}
-
-
-/**
- * swap_fences_unref - empty the throttle queue
- *
- * pulls fences of the throttle queue until it is empty.
- */
-static void
-swap_fences_unref(struct dri_drawable *draw)
-{
-   struct pipe_screen *screen = draw->screen->base.screen;
-
-   while(draw->cur_fences) {
-      screen->fence_reference(screen, &draw->swap_fences[draw->tail++], NULL);
-      draw->tail &= DRI_SWAP_FENCES_MASK;
-      --draw->cur_fences;
-   }
-}
-
 void
 dri_pipe_blit(struct pipe_context *pipe,
               struct pipe_resource *dst,
@@ -475,6 +404,56 @@ dri_postprocessing(struct dri_context *ctx,
       pp_run(ctx->pp, src, src, zsbuf);
 }
 
+struct notify_before_flush_cb_args {
+   struct dri_context *ctx;
+   struct dri_drawable *drawable;
+   unsigned flags;
+   enum __DRI2throttleReason reason;
+   bool swap_msaa_buffers;
+};
+
+static void
+notify_before_flush_cb(void* _args)
+{
+   struct notify_before_flush_cb_args *args = (struct notify_before_flush_cb_args *) _args;
+   struct st_context_iface *st = args->ctx->st;
+   struct pipe_context *pipe = st->pipe;
+
+   if (args->drawable->stvis.samples > 1 &&
+       (args->reason == __DRI2_THROTTLE_SWAPBUFFER ||
+        args->reason == __DRI2_THROTTLE_COPYSUBBUFFER)) {
+      /* Resolve the MSAA back buffer. */
+      dri_pipe_blit(st->pipe,
+                    args->drawable->textures[ST_ATTACHMENT_BACK_LEFT],
+                    args->drawable->msaa_textures[ST_ATTACHMENT_BACK_LEFT]);
+
+      if (args->reason == __DRI2_THROTTLE_SWAPBUFFER &&
+          args->drawable->msaa_textures[ST_ATTACHMENT_FRONT_LEFT] &&
+          args->drawable->msaa_textures[ST_ATTACHMENT_BACK_LEFT]) {
+         args->swap_msaa_buffers = true;
+      }
+
+      /* FRONT_LEFT is resolved in drawable->flush_frontbuffer. */
+   }
+
+   dri_postprocessing(args->ctx, args->drawable, ST_ATTACHMENT_BACK_LEFT);
+
+   if (pipe->invalidate_resource &&
+       (args->flags & __DRI2_FLUSH_INVALIDATE_ANCILLARY)) {
+      if (args->drawable->textures[ST_ATTACHMENT_DEPTH_STENCIL])
+         pipe->invalidate_resource(pipe, args->drawable->textures[ST_ATTACHMENT_DEPTH_STENCIL]);
+      if (args->drawable->msaa_textures[ST_ATTACHMENT_DEPTH_STENCIL])
+         pipe->invalidate_resource(pipe, args->drawable->msaa_textures[ST_ATTACHMENT_DEPTH_STENCIL]);
+   }
+
+   if (args->ctx->hud) {
+      hud_run(args->ctx->hud, args->ctx->st->cso_context,
+              args->drawable->textures[ST_ATTACHMENT_BACK_LEFT]);
+   }
+
+   pipe->flush_resource(pipe, args->drawable->textures[ST_ATTACHMENT_BACK_LEFT]);
+}
+
 /**
  * DRI2 flush extension, the flush_with_flags function.
  *
@@ -493,7 +472,7 @@ dri_flush(__DRIcontext *cPriv,
    struct dri_drawable *drawable = dri_drawable(dPriv);
    struct st_context_iface *st;
    unsigned flush_flags;
-   bool swap_msaa_buffers = false;
+   struct notify_before_flush_cb_args args = { 0 };
 
    if (!ctx) {
       assert(0);
@@ -515,42 +494,18 @@ dri_flush(__DRIcontext *cPriv,
       flags &= ~__DRI2_FLUSH_DRAWABLE;
    }
 
-   /* Flush the drawable. */
    if ((flags & __DRI2_FLUSH_DRAWABLE) &&
        drawable->textures[ST_ATTACHMENT_BACK_LEFT]) {
-      struct pipe_context *pipe = st->pipe;
-
-      if (drawable->stvis.samples > 1 &&
-          reason == __DRI2_THROTTLE_SWAPBUFFER) {
-         /* Resolve the MSAA back buffer. */
-         dri_pipe_blit(st->pipe,
-                       drawable->textures[ST_ATTACHMENT_BACK_LEFT],
-                       drawable->msaa_textures[ST_ATTACHMENT_BACK_LEFT]);
-
-         if (drawable->msaa_textures[ST_ATTACHMENT_FRONT_LEFT] &&
-             drawable->msaa_textures[ST_ATTACHMENT_BACK_LEFT]) {
-            swap_msaa_buffers = true;
-         }
-
-         /* FRONT_LEFT is resolved in drawable->flush_frontbuffer. */
-      }
-
-      dri_postprocessing(ctx, drawable, ST_ATTACHMENT_BACK_LEFT);
-
-      if (pipe->invalidate_resource &&
-          (flags & __DRI2_FLUSH_INVALIDATE_ANCILLARY)) {
-         if (drawable->textures[ST_ATTACHMENT_DEPTH_STENCIL])
-            pipe->invalidate_resource(pipe, drawable->textures[ST_ATTACHMENT_DEPTH_STENCIL]);
-         if (drawable->msaa_textures[ST_ATTACHMENT_DEPTH_STENCIL])
-            pipe->invalidate_resource(pipe, drawable->msaa_textures[ST_ATTACHMENT_DEPTH_STENCIL]);
-      }
-
-      if (ctx->hud) {
-         hud_run(ctx->hud, ctx->st->cso_context,
-                 drawable->textures[ST_ATTACHMENT_BACK_LEFT]);
-      }
-
-      pipe->flush_resource(pipe, drawable->textures[ST_ATTACHMENT_BACK_LEFT]);
+      /* We can't do operations on the back buffer here, because there
+       * may be some pending operations that will get flushed by the
+       * call to st->flush (eg: FLUSH_VERTICES).
+       * Instead we register a callback to be notified when all operations
+       * have been submitted but before the call to st_flush.
+       */
+      args.ctx = ctx;
+      args.drawable = drawable;
+      args.flags = flags;
+      args.reason = reason;
    }
 
    flush_flags = 0;
@@ -560,38 +515,25 @@ dri_flush(__DRIcontext *cPriv,
       flush_flags |= ST_FLUSH_END_OF_FRAME;
 
    /* Flush the context and throttle if needed. */
-   if (dri_screen(ctx->sPriv)->default_throttle_frames &&
+   if (dri_screen(ctx->sPriv)->throttle &&
        drawable &&
        (reason == __DRI2_THROTTLE_SWAPBUFFER ||
         reason == __DRI2_THROTTLE_FLUSHFRONT)) {
-      /* Throttle.
-       *
-       * This pulls a fence off the throttling queue and waits for it if the
-       * number of fences on the throttling queue has reached the desired
-       * number.
-       *
-       * Then flushes to insert a fence at the current rendering position, and
-       * pushes that fence on the queue. This requires that the st_context_iface
-       * flush method returns a fence even if there are no commands to flush.
-       */
+
       struct pipe_screen *screen = drawable->screen->base.screen;
-      struct pipe_fence_handle *oldest_fence, *new_fence = NULL;
+      struct pipe_fence_handle *new_fence = NULL;
 
-      st->flush(st, flush_flags, &new_fence);
+      st->flush(st, flush_flags, &new_fence, args.ctx ? notify_before_flush_cb : NULL, &args);
 
-      oldest_fence = swap_fences_pop_front(drawable);
-      if (oldest_fence) {
-         screen->fence_finish(screen, NULL, oldest_fence, PIPE_TIMEOUT_INFINITE);
-         screen->fence_reference(screen, &oldest_fence, NULL);
-      }
-
-      if (new_fence) {
-         swap_fences_push_back(drawable, new_fence);
-         screen->fence_reference(screen, &new_fence, NULL);
+      /* throttle on the previous fence */
+      if (drawable->throttle_fence) {
+         screen->fence_finish(screen, NULL, drawable->throttle_fence, PIPE_TIMEOUT_INFINITE);
+         screen->fence_reference(screen, &drawable->throttle_fence, NULL);
       }
+      drawable->throttle_fence = new_fence;
    }
    else if (flags & (__DRI2_FLUSH_DRAWABLE | __DRI2_FLUSH_CONTEXT)) {
-      st->flush(st, flush_flags, NULL);
+      st->flush(st, flush_flags, NULL, args.ctx ? notify_before_flush_cb : NULL, &args);
    }
 
    if (drawable) {
@@ -602,7 +544,7 @@ dri_flush(__DRIcontext *cPriv,
     * from the front buffer after SwapBuffers returns what was
     * in the back buffer.
     */
-   if (swap_msaa_buffers) {
+   if (args.swap_msaa_buffers) {
       struct pipe_resource *tmp =
          drawable->msaa_textures[ST_ATTACHMENT_FRONT_LEFT];
 
