@@ -235,10 +235,10 @@ static OMX_ERRORTYPE vid_enc_Constructor(OMX_COMPONENTTYPE *comp, OMX_STRING nam
    priv->scale.xWidth = OMX_VID_ENC_SCALING_WIDTH_DEFAULT;
    priv->scale.xHeight = OMX_VID_ENC_SCALING_WIDTH_DEFAULT;
 
-   LIST_INITHEAD(&priv->free_tasks);
-   LIST_INITHEAD(&priv->used_tasks);
-   LIST_INITHEAD(&priv->b_frames);
-   LIST_INITHEAD(&priv->stacked_tasks);
+   list_inithead(&priv->free_tasks);
+   list_inithead(&priv->used_tasks);
+   list_inithead(&priv->b_frames);
+   list_inithead(&priv->stacked_tasks);
 
    return OMX_ErrorNone;
 }
@@ -658,7 +658,7 @@ static OMX_ERRORTYPE vid_enc_AllocateInBuffer(omx_base_PortType *port, OMX_INOUT
       return OMX_ErrorInsufficientResources;
    }
 
-   LIST_INITHEAD(&inp->tasks);
+   list_inithead(&inp->tasks);
 
    FREE((*buf)->pBuffer);
    r = enc_AllocateBackTexture(port, &inp->resource, &inp->transfer, &(*buf)->pBuffer);
@@ -687,7 +687,7 @@ static OMX_ERRORTYPE vid_enc_UseInBuffer(omx_base_PortType *port, OMX_BUFFERHEAD
       return OMX_ErrorInsufficientResources;
    }
 
-   LIST_INITHEAD(&inp->tasks);
+   list_inithead(&inp->tasks);
 
    return OMX_ErrorNone;
 }
@@ -820,16 +820,16 @@ static void enc_ClearBframes(omx_base_PortType *port, struct input_buf_private *
    vid_enc_PrivateType *priv = comp->pComponentPrivate;
    struct encode_task *task;
 
-   if (LIST_IS_EMPTY(&priv->b_frames))
+   if (list_is_empty(&priv->b_frames))
       return;
 
    task = LIST_ENTRY(struct encode_task, priv->b_frames.prev, list);
-   LIST_DEL(&task->list);
+   list_del(&task->list);
 
    /* promote last from to P frame */
    priv->ref_idx_l0 = priv->ref_idx_l1;
    enc_HandleTask(port, task, PIPE_H264_ENC_PICTURE_TYPE_P);
-   LIST_ADDTAIL(&task->list, &inp->tasks);
+   list_addtail(&task->list, &inp->tasks);
    priv->ref_idx_l1 = priv->frame_num++;
 
    /* handle B frames */
@@ -900,20 +900,20 @@ static OMX_ERRORTYPE vid_enc_EncodeFrame(omx_base_PortType *port, OMX_BUFFERHEAD
 
    if (picture_type == PIPE_H264_ENC_PICTURE_TYPE_B) {
       /* put frame at the tail of the queue */
-      LIST_ADDTAIL(&task->list, &priv->b_frames);
+      list_addtail(&task->list, &priv->b_frames);
    } else {
       /* handle I or P frame */
       priv->ref_idx_l0 = priv->ref_idx_l1;
       enc_HandleTask(port, task, picture_type);
-      LIST_ADDTAIL(&task->list, &priv->stacked_tasks);
+      list_addtail(&task->list, &priv->stacked_tasks);
       LIST_FOR_EACH_ENTRY(task, &priv->stacked_tasks, list) {
          ++stacked_num;
       }
       if (stacked_num == priv->stacked_frames_num) {
          struct encode_task *t;
          t = LIST_ENTRY(struct encode_task, priv->stacked_tasks.next, list);
-         LIST_DEL(&t->list);
-         LIST_ADDTAIL(&t->list, &inp->tasks);
+         list_del(&t->list);
+         list_addtail(&t->list, &inp->tasks);
       }
       priv->ref_idx_l1 = priv->frame_num++;
 
@@ -928,7 +928,7 @@ static OMX_ERRORTYPE vid_enc_EncodeFrame(omx_base_PortType *port, OMX_BUFFERHEAD
       enc_MoveTasks(&priv->b_frames, &inp->tasks);
    }
 
-   if (LIST_IS_EMPTY(&inp->tasks))
+   if (list_is_empty(&inp->tasks))
       return port->ReturnBufferFunction(port, buf);
    else
       return base_port_SendBufferFunction(port, buf);
