@@ -50,6 +50,9 @@
 #include "glxextensions.h"
 
 #include "util/debug.h"
+#ifndef GLX_USE_APPLEGL
+#include "dri_common.h"
+#endif
 
 #include <X11/Xlib-xcb.h>
 #include <xcb/xcb.h>
@@ -907,6 +910,11 @@ __glXInitialize(Display * dpy)
 
    dpyPriv->drawHash = __glxHashCreate();
 
+#ifndef GLX_USE_APPLEGL
+   /* Set the logger before the *CreateDisplay functions. */
+   loader_set_logger(dri_message);
+#endif
+
    /*
     ** Initialize the direct rendering per display data and functions.
     ** Note: This _must_ be done before calling any other DRI routines
