@@ -1053,7 +1053,7 @@ offsets = {
     "TextureSubImage3D": 1017,
     "TransformFeedbackBufferBase": 1018,
     "TransformFeedbackBufferRange": 1019,
-    "UnmapNamedBuffer": 1020,
+    "UnmapNamedBufferEXT": 1020,
     "VertexArrayAttribBinding": 1021,
     "VertexArrayAttribFormat": 1022,
     "VertexArrayAttribIFormat": 1023,
@@ -1453,6 +1453,193 @@ offsets = {
     "TexParameterxv": 1417,
     "BlendBarrier": 1418,
     "PrimitiveBoundingBox": 1419,
+    "MaxShaderCompilerThreadsKHR": 1420,
+    "MatrixLoadfEXT": 1421,
+    "MatrixLoaddEXT": 1422,
+    "MatrixMultfEXT": 1423,
+    "MatrixMultdEXT": 1424,
+    "MatrixLoadIdentityEXT": 1425,
+    "MatrixRotatefEXT": 1426,
+    "MatrixRotatedEXT": 1427,
+    "MatrixScalefEXT": 1428,
+    "MatrixScaledEXT": 1429,
+    "MatrixTranslatefEXT": 1430,
+    "MatrixTranslatedEXT": 1431,
+    "MatrixOrthoEXT": 1432,
+    "MatrixFrustumEXT": 1433,
+    "MatrixPushEXT": 1434,
+    "MatrixPopEXT": 1435,
+    "MatrixLoadTransposefEXT": 1436,
+    "MatrixLoadTransposedEXT": 1437,
+    "MatrixMultTransposefEXT": 1438,
+    "MatrixMultTransposedEXT": 1439,
+    "BindMultiTextureEXT": 1440,
+    "NamedBufferDataEXT": 1441,
+    "NamedBufferSubDataEXT": 1442,
+    "NamedBufferStorageEXT": 1443,
+    "MapNamedBufferRangeEXT": 1444,
+    "TextureImage1DEXT": 1445,
+    "TextureImage2DEXT": 1446,
+    "TextureImage3DEXT": 1447,
+    "TextureSubImage1DEXT": 1448,
+    "TextureSubImage2DEXT": 1449,
+    "TextureSubImage3DEXT": 1450,
+    "CopyTextureImage1DEXT": 1451,
+    "CopyTextureImage2DEXT": 1452,
+    "CopyTextureSubImage1DEXT": 1453,
+    "CopyTextureSubImage2DEXT": 1454,
+    "CopyTextureSubImage3DEXT": 1455,
+    "MapNamedBufferEXT": 1456,
+    "GetTextureParameterivEXT": 1457,
+    "GetTextureParameterfvEXT": 1458,
+    "TextureParameteriEXT": 1459,
+    "TextureParameterivEXT": 1460,
+    "TextureParameterfEXT": 1461,
+    "TextureParameterfvEXT": 1462,
+    "GetTextureImageEXT": 1463,
+    "GetTextureLevelParameterivEXT": 1464,
+    "GetTextureLevelParameterfvEXT": 1465,
+    "GetNamedBufferSubDataEXT": 1466,
+    "GetNamedBufferPointervEXT": 1467,
+    "GetNamedBufferParameterivEXT": 1468,
+    "FlushMappedNamedBufferRangeEXT": 1469,
+    "FramebufferDrawBufferEXT": 1470,
+    "FramebufferDrawBuffersEXT": 1471,
+    "FramebufferReadBufferEXT": 1472,
+    "GetFramebufferParameterivEXT": 1473,
+    "CheckNamedFramebufferStatusEXT": 1474,
+    "NamedFramebufferTexture1DEXT": 1475,
+    "NamedFramebufferTexture2DEXT": 1476,
+    "NamedFramebufferTexture3DEXT": 1477,
+    "NamedFramebufferRenderbufferEXT": 1478,
+    "GetNamedFramebufferAttachmentParameterivEXT": 1479,
+    "EnableClientStateiEXT": 1480,
+    "DisableClientStateiEXT": 1481,
+    "GetPointerIndexedvEXT": 1482,
+    "MultiTexEnviEXT": 1483,
+    "MultiTexEnvivEXT": 1484,
+    "MultiTexEnvfEXT": 1485,
+    "MultiTexEnvfvEXT": 1486,
+    "GetMultiTexEnvivEXT": 1487,
+    "GetMultiTexEnvfvEXT": 1488,
+    "MultiTexParameteriEXT": 1489,
+    "MultiTexParameterivEXT": 1490,
+    "MultiTexParameterfEXT": 1491,
+    "MultiTexParameterfvEXT": 1492,
+    "GetMultiTexImageEXT": 1493,
+    "MultiTexImage1DEXT": 1494,
+    "MultiTexImage2DEXT": 1495,
+    "MultiTexImage3DEXT": 1496,
+    "MultiTexSubImage1DEXT": 1497,
+    "MultiTexSubImage2DEXT": 1498,
+    "MultiTexSubImage3DEXT": 1499,
+    "GetMultiTexParameterivEXT": 1500,
+    "GetMultiTexParameterfvEXT": 1501,
+    "CopyMultiTexImage1DEXT": 1502,
+    "CopyMultiTexImage2DEXT": 1503,
+    "CopyMultiTexSubImage1DEXT": 1504,
+    "CopyMultiTexSubImage2DEXT": 1505,
+    "CopyMultiTexSubImage3DEXT": 1506,
+    "MultiTexGendEXT": 1507,
+    "MultiTexGendvEXT": 1508,
+    "MultiTexGenfEXT": 1509,
+    "MultiTexGenfvEXT": 1510,
+    "MultiTexGeniEXT": 1511,
+    "MultiTexGenivEXT": 1512,
+    "GetMultiTexGendvEXT": 1513,
+    "GetMultiTexGenfvEXT": 1514,
+    "GetMultiTexGenivEXT": 1515,
+    "MultiTexCoordPointerEXT": 1516,
+    "BindImageTextureEXT": 1517,
+    "CompressedTextureImage1DEXT": 1518,
+    "CompressedTextureImage2DEXT": 1519,
+    "CompressedTextureImage3DEXT": 1520,
+    "CompressedTextureSubImage1DEXT": 1521,
+    "CompressedTextureSubImage2DEXT": 1522,
+    "CompressedTextureSubImage3DEXT": 1523,
+    "GetCompressedTextureImageEXT": 1524,
+    "CompressedMultiTexImage1DEXT": 1525,
+    "CompressedMultiTexImage2DEXT": 1526,
+    "CompressedMultiTexImage3DEXT": 1527,
+    "CompressedMultiTexSubImage1DEXT": 1528,
+    "CompressedMultiTexSubImage2DEXT": 1529,
+    "CompressedMultiTexSubImage3DEXT": 1530,
+    "GetCompressedMultiTexImageEXT": 1531,
+    "GetMultiTexLevelParameterivEXT": 1532,
+    "GetMultiTexLevelParameterfvEXT": 1533,
+    "FramebufferParameteriMESA": 1534,
+    "GetFramebufferParameterivMESA": 1535,
+    "NamedRenderbufferStorageEXT": 1536,
+    "GetNamedRenderbufferParameterivEXT": 1537,
+    "ClientAttribDefaultEXT": 1538,
+    "PushClientAttribDefaultEXT": 1539,
+    "NamedProgramStringEXT": 1540,
+    "GetNamedProgramStringEXT": 1541,
+    "NamedProgramLocalParameter4fEXT": 1542,
+    "NamedProgramLocalParameter4fvEXT": 1543,
+    "GetNamedProgramLocalParameterfvEXT": 1544,
+    "NamedProgramLocalParameter4dEXT": 1545,
+    "NamedProgramLocalParameter4dvEXT": 1546,
+    "GetNamedProgramLocalParameterdvEXT": 1547,
+    "GetNamedProgramivEXT": 1548,
+    "TextureBufferEXT": 1549,
+    "MultiTexBufferEXT": 1550,
+    "TextureParameterIivEXT": 1551,
+    "TextureParameterIuivEXT": 1552,
+    "GetTextureParameterIivEXT": 1553,
+    "GetTextureParameterIuivEXT": 1554,
+    "MultiTexParameterIivEXT": 1555,
+    "MultiTexParameterIuivEXT": 1556,
+    "GetMultiTexParameterIivEXT": 1557,
+    "GetMultiTexParameterIuivEXT": 1558,
+    "NamedProgramLocalParameters4fvEXT": 1559,
+    "GenerateTextureMipmapEXT": 1560,
+    "GenerateMultiTexMipmapEXT": 1561,
+    "NamedRenderbufferStorageMultisampleEXT": 1562,
+    "NamedCopyBufferSubDataEXT": 1563,
+    "VertexArrayVertexOffsetEXT": 1564,
+    "VertexArrayColorOffsetEXT": 1565,
+    "VertexArrayEdgeFlagOffsetEXT": 1566,
+    "VertexArrayIndexOffsetEXT": 1567,
+    "VertexArrayNormalOffsetEXT": 1568,
+    "VertexArrayTexCoordOffsetEXT": 1569,
+    "VertexArrayMultiTexCoordOffsetEXT": 1570,
+    "VertexArrayFogCoordOffsetEXT": 1571,
+    "VertexArraySecondaryColorOffsetEXT": 1572,
+    "VertexArrayVertexAttribOffsetEXT": 1573,
+    "VertexArrayVertexAttribIOffsetEXT": 1574,
+    "EnableVertexArrayEXT": 1575,
+    "DisableVertexArrayEXT": 1576,
+    "EnableVertexArrayAttribEXT": 1577,
+    "DisableVertexArrayAttribEXT": 1578,
+    "GetVertexArrayIntegervEXT": 1579,
+    "GetVertexArrayPointervEXT": 1580,
+    "GetVertexArrayIntegeri_vEXT": 1581,
+    "GetVertexArrayPointeri_vEXT": 1582,
+    "ClearNamedBufferDataEXT": 1583,
+    "ClearNamedBufferSubDataEXT": 1584,
+    "NamedFramebufferParameteriEXT": 1585,
+    "GetNamedFramebufferParameterivEXT": 1586,
+    "VertexArrayVertexAttribLOffsetEXT": 1587,
+    "VertexArrayVertexAttribDivisorEXT": 1588,
+    "TextureBufferRangeEXT": 1589,
+    "TextureStorage2DMultisampleEXT": 1590,
+    "TextureStorage3DMultisampleEXT": 1591,
+    "VertexArrayBindVertexBufferEXT": 1592,
+    "VertexArrayVertexAttribFormatEXT": 1593,
+    "VertexArrayVertexAttribIFormatEXT": 1594,
+    "VertexArrayVertexAttribLFormatEXT": 1595,
+    "VertexArrayVertexAttribBindingEXT": 1596,
+    "VertexArrayVertexBindingDivisorEXT": 1597,
+    "NamedBufferPageCommitmentEXT": 1598,
+    "NamedStringARB": 1599,
+    "DeleteNamedStringARB": 1600,
+    "CompileShaderIncludeARB": 1601,
+    "IsNamedStringARB": 1602,
+    "GetNamedStringARB": 1603,
+    "GetNamedStringivARB": 1604,
+    "EGLImageTargetTexStorageEXT" : 1605,
+    "EGLImageTargetTextureStorageEXT" : 1606,
 }
 
 functions = [
