@@ -34,9 +34,9 @@
 #include "main/mtypes.h"
 #include "main/errors.h"
 
-#include <drm.h>
+#include "drm-uapi/drm.h"
 #include <intel_bufmgr.h>
-#include <i915_drm.h>
+#include "drm-uapi/i915_drm.h"
 
 #include "intel_screen.h"
 #include "intel_tex_obj.h"
@@ -159,7 +159,6 @@ struct intel_context
     */
    int gen;
    bool is_945;
-   bool has_swizzling;
 
    struct intel_batchbuffer batch;
 
@@ -238,8 +237,6 @@ struct intel_context
     */
    bool front_buffer_dirty;
 
-   bool use_early_z;
-
    __DRIcontext *driContext;
    struct intel_screen *intelScreen;
 
@@ -307,11 +304,11 @@ extern int INTEL_DEBUG;
    if (unlikely(INTEL_DEBUG & DEBUG_PERF))                      \
       dbg_printf(__VA_ARGS__);                                  \
    if (intel->perf_debug)                                       \
-      _mesa_gl_debug(&intel->ctx, &msg_id,                      \
-                     MESA_DEBUG_SOURCE_API,                     \
-                     MESA_DEBUG_TYPE_PERFORMANCE,               \
-                     MESA_DEBUG_SEVERITY_MEDIUM,                \
-                     __VA_ARGS__);                              \
+      _mesa_gl_debugf(&intel->ctx, &msg_id,                     \
+                      MESA_DEBUG_SOURCE_API,                    \
+                      MESA_DEBUG_TYPE_PERFORMANCE,              \
+                      MESA_DEBUG_SEVERITY_MEDIUM,               \
+                      __VA_ARGS__);                             \
 } while(0)
 
 #define WARN_ONCE(cond, fmt...) do {                            \
@@ -323,10 +320,10 @@ extern int INTEL_DEBUG;
          fprintf(stderr, fmt);                                  \
          _warned = true;                                        \
                                                                 \
-         _mesa_gl_debug(ctx, &msg_id,                           \
-                        MESA_DEBUG_SOURCE_API,                  \
-                        MESA_DEBUG_TYPE_OTHER,                  \
-                        MESA_DEBUG_SEVERITY_HIGH, fmt);         \
+         _mesa_gl_debugf(ctx, &msg_id,                          \
+                         MESA_DEBUG_SOURCE_API,                 \
+                         MESA_DEBUG_TYPE_OTHER,                 \
+                         MESA_DEBUG_SEVERITY_HIGH, fmt);        \
       }                                                         \
    }                                                            \
 } while (0)
