@@ -103,32 +103,22 @@ get_bsw_model(const struct intel_screen *screen)
 const char *
 brw_get_renderer_string(const struct intel_screen *screen)
 {
-   const char *chipset;
-   static char buffer[128];
-   char *bsw = NULL;
+   static char buf[128];
+   const char *name = gen_get_device_name(screen->deviceID);
 
-   switch (screen->deviceID) {
-#undef CHIPSET
-#define CHIPSET(id, symbol, str) case id: chipset = str; break;
-#include "pci_ids/i965_pci_ids.h"
-   default:
-      chipset = "Unknown Intel Chipset";
-      break;
-   }
+   if (!name)
+      name = "Intel Unknown";
+
+   snprintf(buf, sizeof(buf), "Mesa DRI %s", name);
 
    /* Braswell branding is funny, so we have to fix it up here */
    if (screen->deviceID == 0x22B1) {
-      bsw = strdup(chipset);
-      char *needle = strstr(bsw, "XXX");
-      if (needle) {
+      char *needle = strstr(buf, "XXX");
+      if (needle)
          memcpy(needle, get_bsw_model(screen), 3);
-         chipset = bsw;
-      }
    }
 
-   (void) driGetRendererString(buffer, chipset, 0);
-   free(bsw);
-   return buffer;
+   return buf;
 }
 
 static const GLubyte *
@@ -421,6 +411,7 @@ brw_initialize_spirv_supported_capabilities(struct brw_context *brw)
    ctx->Const.SpirVCapabilities.tessellation = true;
    ctx->Const.SpirVCapabilities.transform_feedback = devinfo->gen >= 7;
    ctx->Const.SpirVCapabilities.variable_pointers = true;
+   ctx->Const.SpirVCapabilities.integer_functions2 = devinfo->gen >= 8;
 }
 
 static void
@@ -846,15 +837,6 @@ brw_process_driconf_options(struct brw_context *brw)
                        brw->driContext->driScreenPriv->myNum,
                        "i965", NULL, NULL, 0);
 
-   int bo_reuse_mode = driQueryOptioni(options, "bo_reuse");
-   switch (bo_reuse_mode) {
-   case DRI_CONF_BO_REUSE_DISABLED:
-      break;
-   case DRI_CONF_BO_REUSE_ALL:
-      brw_bufmgr_enable_reuse(brw->bufmgr);
-      break;
-   }
-
    if (INTEL_DEBUG & DEBUG_NO_HIZ) {
        brw->has_hiz = false;
        /* On gen6, you can only do separate stencil with HIZ. */
@@ -1236,7 +1218,7 @@ intelDestroyContext(__DRIcontext * driContextPriv)
    driDestroyOptionCache(&brw->optionCache);
 
    /* free the Mesa context */
-   _mesa_free_context_data(&brw->ctx, true);
+   _mesa_free_context_data(&brw->ctx);
 
    ralloc_free(brw);
    driContextPriv->driverPrivate = NULL;
@@ -1541,8 +1523,10 @@ intel_prepare_render(struct brw_context *brw)
     * that will happen next will probably dirty the front buffer.  So
     * mark it as dirty here.
     */
-   if (_mesa_is_front_buffer_drawing(ctx->DrawBuffer))
+   if (_mesa_is_front_buffer_drawing(ctx->DrawBuffer) &&
+       ctx->DrawBuffer != _mesa_get_incomplete_framebuffer()) {
       brw->front_buffer_dirty = true;
+   }
 
    if (brw->is_shared_buffer_bound) {
       /* Subsequent rendering will probably dirty the shared buffer. */
