@@ -88,6 +88,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       break;
    case GL_RGB5_A1:
       RETURN_IF_SUPPORTED(MESA_FORMAT_B5G5R5A1_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_A1B5G5R5_UNORM);
       break;
    case GL_RGBA2:
       RETURN_IF_SUPPORTED(MESA_FORMAT_A4R4G4B4_UNORM); /* just to test another format*/
@@ -117,6 +118,9 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       if (type == GL_UNSIGNED_INT_2_10_10_10_REV) {
          RETURN_IF_SUPPORTED(MESA_FORMAT_B10G10R10A2_UNORM);
       }
+      if (type == GL_UNSIGNED_SHORT_5_6_5) {
+         RETURN_IF_SUPPORTED(MESA_FORMAT_B5G6R5_UNORM);
+      }
       /* fallthrough */
    case GL_RGB8:
       RETURN_IF_SUPPORTED(MESA_FORMAT_RGB_UNORM8);
@@ -158,14 +162,14 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
    case GL_ALPHA4:
    case GL_ALPHA8:
       RETURN_IF_SUPPORTED(MESA_FORMAT_A_UNORM8);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
 
    case GL_ALPHA12:
    case GL_ALPHA16:
       RETURN_IF_SUPPORTED(MESA_FORMAT_A_UNORM16);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A_UNORM8);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
 
    /* Luminance formats */
@@ -185,21 +189,21 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       /* Luminance/Alpha formats */
    case GL_LUMINANCE4_ALPHA4:
       RETURN_IF_SUPPORTED(MESA_FORMAT_L4A4_UNORM);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
 
    case 2:
    case GL_LUMINANCE_ALPHA:
    case GL_LUMINANCE6_ALPHA2:
    case GL_LUMINANCE8_ALPHA8:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
 
    case GL_LUMINANCE12_ALPHA4:
    case GL_LUMINANCE12_ALPHA12:
    case GL_LUMINANCE16_ALPHA16:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L16A16_UNORM);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM16);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
 
    case GL_INTENSITY:
@@ -229,13 +233,13 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
 
    case GL_COMPRESSED_ALPHA_ARB:
       RETURN_IF_SUPPORTED(MESA_FORMAT_A_UNORM8);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
    case GL_COMPRESSED_LUMINANCE_ARB:
       RETURN_IF_SUPPORTED(MESA_FORMAT_L_UNORM8);
       break;
    case GL_COMPRESSED_LUMINANCE_ALPHA_ARB:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_UNORM8);
       break;
    case GL_COMPRESSED_INTENSITY_ARB:
       RETURN_IF_SUPPORTED(MESA_FORMAT_I_UNORM8);
@@ -249,9 +253,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
        * 1D ARRAY textures in S3TC format.
        */
       if (target != GL_TEXTURE_1D && target != GL_TEXTURE_1D_ARRAY) {
-         if (ctx->Mesa_DXTn)
-            RETURN_IF_SUPPORTED(MESA_FORMAT_RGB_DXT1);
-         RETURN_IF_SUPPORTED(MESA_FORMAT_RGB_FXT1);
+         RETURN_IF_SUPPORTED(MESA_FORMAT_RGB_DXT1);
       }
       RETURN_IF_SUPPORTED(MESA_FORMAT_BGR_UNORM8);
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8X8_UNORM);
@@ -260,9 +262,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
    case GL_COMPRESSED_RGBA_ARB:
       /* We don't use texture compression for 1D and 1D array textures. */
       if (target != GL_TEXTURE_1D && target != GL_TEXTURE_1D_ARRAY) {
-         if (ctx->Mesa_DXTn)
-            RETURN_IF_SUPPORTED(MESA_FORMAT_RGBA_DXT3); /* Not rgba_dxt1, see spec */
-         RETURN_IF_SUPPORTED(MESA_FORMAT_RGBA_FXT1);
+         RETURN_IF_SUPPORTED(MESA_FORMAT_RGBA_DXT3); /* Not rgba_dxt1, see spec */
       }
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8B8G8R8_UNORM);
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8A8_UNORM);
@@ -379,7 +379,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       break;
    case GL_RG_SNORM:
    case GL_RG8_SNORM:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_R8G8_SNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_RG_SNORM8);
       break;
    case GL_RGB_SNORM:
    case GL_RGB8_SNORM:
@@ -409,8 +409,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       break;
    case GL_LUMINANCE_ALPHA_SNORM:
    case GL_LUMINANCE8_ALPHA8_SNORM:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_SNORM);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_A8L8_SNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_SNORM8);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8B8G8R8_SNORM);
       RETURN_IF_SUPPORTED(MESA_FORMAT_R8G8B8A8_SNORM);
       break;
@@ -424,7 +423,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       RETURN_IF_SUPPORTED(MESA_FORMAT_R_SNORM16);
       break;
    case GL_RG16_SNORM:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_R16G16_SNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_RG_SNORM16);
       break;
    case GL_RGB16_SNORM:
       RETURN_IF_SUPPORTED(MESA_FORMAT_RGB_SNORM16);
@@ -477,6 +476,9 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8A8_SRGB);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8R8G8B8_SRGB);
       break;
+   case GL_SR8_EXT:
+      RETURN_IF_SUPPORTED(MESA_FORMAT_R_SRGB8);
+      break;
    case GL_SLUMINANCE_EXT:
    case GL_SLUMINANCE8_EXT:
       RETURN_IF_SUPPORTED(MESA_FORMAT_L_SRGB8);
@@ -485,8 +487,7 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       break;
    case GL_SLUMINANCE_ALPHA_EXT:
    case GL_SLUMINANCE8_ALPHA8_EXT:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_SRGB);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_A8L8_SRGB);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_SRGB8);
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8A8_SRGB);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8R8G8B8_SRGB);
       break;
@@ -496,21 +497,18 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8R8G8B8_SRGB);
       break;
    case GL_COMPRESSED_SLUMINANCE_ALPHA_EXT:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_L8A8_SRGB);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_A8L8_SRGB);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_LA_SRGB8);
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8A8_SRGB);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8R8G8B8_SRGB);
       break;
    case GL_COMPRESSED_SRGB_EXT:
-      if (ctx->Mesa_DXTn)
-         RETURN_IF_SUPPORTED(MESA_FORMAT_SRGB_DXT1);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_SRGB_DXT1);
       RETURN_IF_SUPPORTED(MESA_FORMAT_BGR_SRGB8);
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8A8_SRGB);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8R8G8B8_SRGB);
       break;
    case GL_COMPRESSED_SRGB_ALPHA_EXT:
-      if (ctx->Mesa_DXTn)
-         RETURN_IF_SUPPORTED(MESA_FORMAT_SRGBA_DXT3); /* Not srgba_dxt1, see spec */
+      RETURN_IF_SUPPORTED(MESA_FORMAT_SRGBA_DXT3); /* Not srgba_dxt1, see spec */
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8B8G8R8_SRGB);
       RETURN_IF_SUPPORTED(MESA_FORMAT_B8G8R8A8_SRGB);
       RETURN_IF_SUPPORTED(MESA_FORMAT_A8R8G8B8_SRGB);
@@ -679,17 +677,17 @@ _mesa_choose_tex_format(struct gl_context *ctx, GLenum target,
 
    case GL_RG:
    case GL_RG8:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_R8G8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_RG_UNORM8);
       break;
 
    case GL_COMPRESSED_RG:
       if (target != GL_TEXTURE_1D && target != GL_TEXTURE_1D_ARRAY)
          RETURN_IF_SUPPORTED(MESA_FORMAT_RG_RGTC2_UNORM);
-      RETURN_IF_SUPPORTED(MESA_FORMAT_R8G8_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_RG_UNORM8);
       break;
 
    case GL_RG16:
-      RETURN_IF_SUPPORTED(MESA_FORMAT_R16G16_UNORM);
+      RETURN_IF_SUPPORTED(MESA_FORMAT_RG_UNORM16);
       break;
 
    case GL_R16F:
