@@ -43,7 +43,7 @@
 #include "st_scissor.h"
 #include "st_util.h"
 
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 
 static void
 st_BlitFramebuffer(struct gl_context *ctx,
