@@ -134,8 +134,6 @@ texfetch_funcs[] =
    FETCH_FUNCS(B8G8R8X8_UNORM),
    FETCH_FUNCS(A8R8G8B8_UNORM),
    FETCH_FUNCS(X8R8G8B8_UNORM),
-   FETCH_FUNCS(L16A16_UNORM),
-   FETCH_FUNCS(A16L16_UNORM),
    FETCH_FUNCS(B5G6R5_UNORM),
    FETCH_FUNCS(R5G6B5_UNORM),
    FETCH_FUNCS(B4G4R4A4_UNORM),
@@ -143,14 +141,8 @@ texfetch_funcs[] =
    FETCH_FUNCS(A1B5G5R5_UNORM),
    FETCH_FUNCS(B5G5R5A1_UNORM),
    FETCH_FUNCS(A1R5G5B5_UNORM),
-   FETCH_FUNCS(L8A8_UNORM),
-   FETCH_FUNCS(A8L8_UNORM),
-   FETCH_FUNCS(R8G8_UNORM),
-   FETCH_FUNCS(G8R8_UNORM),
    FETCH_FUNCS(L4A4_UNORM),
    FETCH_FUNCS(B2G3R3_UNORM),
-   FETCH_FUNCS(R16G16_UNORM),
-   FETCH_FUNCS(G16R16_UNORM),
    FETCH_FUNCS(B10G10R10A2_UNORM),
    FETCH_FUNCS(R10G10B10A2_UNORM),
 
@@ -175,10 +167,14 @@ texfetch_funcs[] =
    FETCH_FUNCS(A_UNORM16),
    FETCH_FUNCS(L_UNORM8),
    FETCH_FUNCS(L_UNORM16),
+   FETCH_FUNCS(LA_UNORM8),
+   FETCH_FUNCS(LA_UNORM16),
    FETCH_FUNCS(I_UNORM8),
    FETCH_FUNCS(I_UNORM16),
    FETCH_FUNCS(R_UNORM8),
    FETCH_FUNCS(R_UNORM16),
+   FETCH_FUNCS(RG_UNORM8),
+   FETCH_FUNCS(RG_UNORM16),
    FETCH_FUNCS(BGR_UNORM8),
    FETCH_FUNCS(RGB_UNORM8),
    FETCH_FUNCS(RGBA_UNORM16),
@@ -190,21 +186,20 @@ texfetch_funcs[] =
    FETCH_FUNCS(A8B8G8R8_SNORM),
    FETCH_FUNCS(X8B8G8R8_SNORM),
    FETCH_FUNCS(R8G8B8A8_SNORM),
-   FETCH_FUNCS(R16G16_SNORM),
-   FETCH_FUNCS(R8G8_SNORM),
-   FETCH_FUNCS(L8A8_SNORM),
-   FETCH_FUNCS(A8L8_SNORM),
 
    /* Array signed/normalized formats */
    FETCH_FUNCS(A_SNORM8),
    FETCH_FUNCS(A_SNORM16),
    FETCH_FUNCS(L_SNORM8),
    FETCH_FUNCS(L_SNORM16),
+   FETCH_FUNCS(LA_SNORM8),
+   FETCH_FUNCS(LA_SNORM16),
    FETCH_FUNCS(I_SNORM8),
    FETCH_FUNCS(I_SNORM16),
    FETCH_FUNCS(R_SNORM8),
    FETCH_FUNCS(R_SNORM16),
-   FETCH_FUNCS(LA_SNORM16),
+   FETCH_FUNCS(RG_SNORM8),
+   FETCH_FUNCS(RG_SNORM16),
    FETCH_FUNCS(RGB_SNORM16),
    FETCH_FUNCS(RGBA_SNORM16),
 
@@ -215,12 +210,11 @@ texfetch_funcs[] =
    FETCH_FUNCS(R8G8B8A8_SRGB),
    FETCH_FUNCS(R8G8B8X8_SRGB),
    FETCH_FUNCS(X8B8G8R8_SRGB),
-   FETCH_FUNCS(L8A8_SRGB),
-   FETCH_FUNCS(A8L8_SRGB),
 
    /* Array sRGB formats */
    FETCH_FUNCS(R_SRGB8),
    FETCH_FUNCS(L_SRGB8),
+   FETCH_FUNCS(LA_SRGB8),
    FETCH_FUNCS(BGR_SRGB8),
 
    /* Packed float formats */
@@ -256,7 +250,6 @@ texfetch_funcs[] =
    /* Packed signed/unsigned non-normalized integer formats */
 
    /* Array signed/unsigned non-normalized integer formats */
-   FETCH_FUNCS(RGBA_UINT8),
    FETCH_FUNCS(RGBA_UINT16),
    FETCH_FUNCS(RGBA_UINT32),
    FETCH_FUNCS(RGBA_SINT8),
